/*****************************************************************************/
/*                        WiNRADiO G39DDC EXCELSIOR                          */
/*                                SDK DEMO                                   */
/*                             Main GUI window                               */
/*                        (C) Copyright RADIXON 2014                         */
/*****************************************************************************/

#include "u_mainwindow.h"
#include "u_defs.h"
#include "u_global.h"


enum
{
    ID_POWER=wxID_HIGHEST,
    ID_SPECTRUM,
    ID_SWEEPER,
    ID_ATTENUATOR,
    ID_PREAMP,
    ID_IFRBW,
    ID_IF_SPECTRUM,
    ID_IF_DOWN,
    ID_IF_UP,
    ID_POWER_STATE_TIMER,
    ID_CONN_TIMER,
    ID_INIT_POWER_ON_TIMER,
    ID_SWP_SPECTRUM,
    ID_SWP_START_FREQ,
    ID_SWP_END_FREQ,
    ID_SWP_START,
    ID_SWP_STOP,
    ID_SWP_LOOP,
    ID_SWP_CLEAR,

    ID_RX_BASE=wxID_HIGHEST+100
};

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(wxEVT_COMMAND_STOP_SWEEPER,5004)
END_DECLARE_EVENT_TYPES()

DEFINE_EVENT_TYPE(wxEVT_COMMAND_STOP_SWEEPER)

#define EVT_STOP_SWEEPER(fn)                                        \
	DECLARE_EVENT_TABLE_ENTRY(wxEVT_COMMAND_STOP_SWEEPER, -1, -1,   \
	(wxObjectEventFunction)(wxEventFunction)                        \
	(wxCommandEventFunction) & fn, (wxObject*) NULL ),

BEGIN_EVENT_TABLE(TMainWindow, wxFrame)
    EVT_TOGGLEBUTTON(ID_POWER,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_PREAMP,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_SPECTRUM,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_SWEEPER,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_SWP_START,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_SWP_LOOP,TMainWindow::OnButtonClick)
    EVT_BUTTON(ID_SWP_STOP,TMainWindow::OnButtonClick)
    EVT_BUTTON(ID_SWP_CLEAR,TMainWindow::OnButtonClick)
    EVT_COMBOBOX(ID_IFRBW,TMainWindow::OnComboBoxChanged)
    EVT_COMBOBOX(ID_ATTENUATOR,TMainWindow::OnComboBoxChanged)
    EVT_SPECTRUM_TUNE(ID_IF_SPECTRUM,TMainWindow::OnIFSpectrumTune)
    EVT_SPECTRUM_TUNE(ID_SWP_SPECTRUM,TMainWindow::OnSwpSpectrumTune)
    EVT_BUTTON(ID_IF_DOWN,TMainWindow::OnButtonClick)
    EVT_BUTTON(ID_IF_UP,TMainWindow::OnButtonClick)
    EVT_TIMER(ID_POWER_STATE_TIMER,TMainWindow::OnTimer)
    EVT_TIMER(ID_CONN_TIMER,TMainWindow::OnTimer)
    EVT_TIMER(ID_INIT_POWER_ON_TIMER,TMainWindow::OnTimer)
    EVT_NUMERICEDIT(ID_SWP_START_FREQ,TMainWindow::OnNumericEditChanged)
    EVT_NUMERICEDIT(ID_SWP_END_FREQ,TMainWindow::OnNumericEditChanged)
    EVT_STOP_SWEEPER(TMainWindow::OnStopSweeper)
END_EVENT_TABLE()


TMainWindow::TMainWindow(TReceiver *Receiver):wxFrame(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition,wxSize(900,800)),
m_Receiver(Receiver),m_PowerStateTimer(this,ID_POWER_STATE_TIMER),
m_ConnTimer(this,ID_CONN_TIMER),m_InitPowerOnTimer(this,ID_INIT_POWER_ON_TIMER)
{
 uint32_t i;
 wxString Title,s;

    m_ActiveRXPanel=NULL;

    m_DeviceInfo=Receiver->GetDeviceInfo();

    SetFont(*Global.m_Font);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));

    Title.sprintf(APP_TITLE wxT(" - %s"),wxString(m_DeviceInfo.SerialNumber,wxConvLocal).c_str());

    SetTitle(Title);

    m_SwpLoop=true;

    m_MainSizer=new wxBoxSizer(wxVERTICAL);

    m_IFSizerTop=new wxBoxSizer(wxHORIZONTAL);
    m_IFSizerBottom=new wxBoxSizer(wxHORIZONTAL);

    m_RXSizer=new wxBoxSizer(wxHORIZONTAL);
    m_MainSizer->Add(m_RXSizer,0,wxEXPAND|wxALIGN_LEFT|wxALIGN_TOP|wxALL,4);

    m_BRX=new wxToggleButton*[m_DeviceInfo.ChannelCount];

    m_RXPanels=new TRXPanel*[m_DeviceInfo.ChannelCount];

    for(i=0;i<m_DeviceInfo.ChannelCount;i++)
    {
        s.sprintf(wxT("RX %u"),i+1);
        m_BRX[i]=new wxToggleButton(this,ID_RX_BASE+i,s,wxDefaultPosition,wxSize(73,23));
        m_RXSizer->Add(m_BRX[i],0,wxRIGHT,4);

        m_RXPanels[i]=new TRXPanel(this,m_Receiver,i);
        m_MainSizer->Add(m_RXPanels[i],1,wxEXPAND|wxLEFT|wxRIGHT,4);

        if(i!=0)
        {
            m_RXPanels[i]->Hide();
        }
    }

    Connect(ID_RX_BASE,ID_RX_BASE+m_DeviceInfo.ChannelCount-1,wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,wxCommandEventHandler(TMainWindow::OnRXButtonClick));

    m_MainSizer->Add(m_IFSizerTop,0,wxALIGN_TOP|wxALL|wxEXPAND,4);

    m_BSpectrum=new wxToggleButton(this,ID_SPECTRUM,wxT("Spectrum"),wxDefaultPosition,wxSize(70,23));
    m_IFSizerTop->Add(m_BSpectrum,0,wxALIGN_LEFT);

    m_IFSizerTop->AddSpacer(4);

    m_BSweeper=new wxToggleButton(this,ID_SWEEPER,wxT("Sweeper"),wxDefaultPosition,wxSize(70,23));
    m_IFSizerTop->Add(m_BSweeper,0,wxALIGN_LEFT);

    m_IFSizerTop->AddSpacer(8);

    m_LRBW=new wxStaticText(this,wxID_ANY,wxT("RBW"));
    m_IFSizerTop->Add(m_LRBW,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(4);

    m_CBIFRBW=new wxComboBox(this,ID_IFRBW,wxEmptyString,wxDefaultPosition,wxSize(75,23),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    m_IFSizerTop->Add(m_CBIFRBW,0,wxALIGN_LEFT|wxCENTRE);

    m_CBSwpRBW=new wxComboBox(this,wxID_ANY,wxEmptyString,wxDefaultPosition,wxSize(75,23),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    m_IFSizerTop->Add(m_CBSwpRBW,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(8);

    m_LSwpStart=new wxStaticText(this,wxID_ANY,wxT("Start frequency"));
    m_IFSizerTop->Add(m_LSwpStart,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(4);

    m_ESwpStart=new TFrequencyEdit(this,ID_SWP_START_FREQ,10000000,m_DeviceInfo.FrontEndMinFrequency-m_DeviceInfo.FrontEndWindowWidth/2,
        m_DeviceInfo.FrontEndMaxFrequency+m_DeviceInfo.FrontEndWindowWidth/2-SWP_STEP,wxDefaultPosition,wxSize(105,23));
    m_IFSizerTop->Add(m_ESwpStart,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(8);

    m_LSwpEnd=new wxStaticText(this,wxID_ANY,wxT("End frequency"));
    m_IFSizerTop->Add(m_LSwpEnd,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(4);

    m_ESwpEnd=new TFrequencyEdit(this,ID_SWP_END_FREQ,100000000,m_DeviceInfo.FrontEndMinFrequency-m_DeviceInfo.FrontEndWindowWidth/2+SWP_STEP,
        m_DeviceInfo.FrontEndMaxFrequency+m_DeviceInfo.FrontEndWindowWidth/2,wxDefaultPosition,wxSize(105,23));
    m_IFSizerTop->Add(m_ESwpEnd,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(8);

    m_BSwpStart=new wxToggleButton(this,ID_SWP_START,wxT("Start"),wxDefaultPosition,wxSize(43,23));
    m_IFSizerTop->Add(m_BSwpStart,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(4);

    m_BSwpStop=new wxButton(this,ID_SWP_STOP,wxT("Stop"),wxDefaultPosition,wxSize(43,23));
    m_IFSizerTop->Add(m_BSwpStop,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(4);

    m_BSwpLoop=new wxToggleButton(this,ID_SWP_LOOP,wxT("Loop"),wxDefaultPosition,wxSize(43,23));
    m_IFSizerTop->Add(m_BSwpLoop,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(8);

    m_BSwpClear=new wxButton(this,ID_SWP_CLEAR,wxT("Clear"),wxDefaultPosition,wxSize(43,23));
    m_IFSizerTop->Add(m_BSwpClear,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddStretchSpacer();

    m_BIFDown=new wxButton(this,ID_IF_DOWN,wxT("<"),wxDefaultPosition,wxSize(30,23));
    m_IFSizerTop->Add(m_BIFDown,0,wxALIGN_RIGHT|wxCENTRE);

    m_IFSizerTop->AddSpacer(4);

    m_BIFUp=new wxButton(this,ID_IF_UP,wxT(">"),wxDefaultPosition,wxSize(30,23));
    m_IFSizerTop->Add(m_BIFUp,0,wxALIGN_RIGHT|wxCENTRE);


    m_IFSpectrum=new TSpectrum(this,ID_IF_SPECTRUM,wxDefaultPosition,wxSize(100,120));
    m_IFSpectrum->SetText(wxT("IF spectrum"));
    m_IFSpectrum->SetMinLevel(-140);
    m_MainSizer->Add(m_IFSpectrum,0,wxEXPAND|wxLEFT|wxRIGHT,4);

    m_SwpSpectrum=new TSwpSpectrum(this,ID_SWP_SPECTRUM,wxDefaultPosition,wxSize(100,120));
    m_MainSizer->Add(m_SwpSpectrum,0,wxEXPAND|wxLEFT|wxRIGHT,4);

    m_MainSizer->Add(m_IFSizerBottom,0,wxALIGN_BOTTOM|wxEXPAND|wxALL,4);

    m_IFSizerBottom->AddSpacer(4);

    m_LAttenuator=new wxStaticText(this,wxID_ANY,wxT("Attenuator"));
    m_IFSizerBottom->Add(m_LAttenuator,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_IFSizerBottom->AddSpacer(4);

    m_CBAttenuator=new wxComboBox(this,ID_ATTENUATOR,wxEmptyString,wxDefaultPosition,wxSize(60,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_IFSizerBottom->Add(m_CBAttenuator,0,wxALIGN_LEFT);

    m_IFSizerBottom->AddSpacer(4);

    m_BPreamp=new wxToggleButton(this,ID_PREAMP,wxT("Preamplifier"),wxDefaultPosition,wxSize(70,23));
    m_IFSizerBottom->Add(m_BPreamp,0,wxALIGN_LEFT);


    m_IFSizerBottom->Add(0,1,1,wxALIGN_LEFT);

    m_BPower=new wxToggleButton(this,ID_POWER,wxT("Power"),wxDefaultPosition,wxSize(70,23));
    m_IFSizerBottom->Add(m_BPower,0,wxRIGHT);


    SetSizer(m_MainSizer);


    Initialize();

    SetClientSize(860,750);
    SetMinSize(wxSize(GetSize().GetWidth(),GetSize().GetHeight()));

    UpdateBottomPanel();

    //set receiver callbacks to this object
    //it is done after Initialize() to prevent from many calls of TReceiver methods
    //during initialization of controls which could invoke many receiver/channel events
    m_Receiver->SetCallback(this);

    m_ConnTimer.Start(500);

    //power on with delay
    m_InitPowerOnTimer.Start(200,true);

    Layout();
    Centre();
}

TMainWindow::~TMainWindow(void)
{
    delete m_Receiver;

    delete[] m_RXPanels;
    delete[] m_BRX;
}

//ReceiverCallback_Event is called when a receiver event occurs (e.g. power state changed, DDC1 frequency changed,...)
void TMainWindow::ReceiverCallback_Event(TReceiverEvent Event)
{
 int64_t Start,End;
 double RBW;

    switch(Event)
    {
        case ReceiverEvent_Power:
            UpdateState();
            break;

        case ReceiverEvent_Attenuator:
            UpdateAttenuatorState();
            break;

        case ReceiverEvent_Preamplifier:
            UpdatePreamplifierState();
            break;

        case ReceiverEvent_FrontEndFrequency:
            UpdateFrontEndFrequency();
            break;

        case ReceiverEvent_HF_UHF:
            UpdateHFUHF();
            break;

        case ReceiverEvent_IFRBWIndex:
            UpdateIFRBWIndex();
            break;

        case ReceiverEvent_ActiveChannel:
            UpdateActiveChannel();
            break;

        case ReceiverEvent_SweeperStarted:
            m_Receiver->GetSweeperParams(&Start,&End,&RBW);
            m_SwpSpectrum->Start(Start,End,RBW);
            UpdateState();
            break;

        case ReceiverEvent_SweeperStopped:
            m_SwpSpectrum->Stop();
            UpdateState();
            break;
    }
}

void TMainWindow::ReceiverCallback_ChannelEvent(uint32_t Channel,TChannelEvent Event)
{
    switch(Event)
    {
        case ChannelEvent_DDC1:
            UpdateDDC1(Channel);
            break;

        case ChannelEvent_DDC1_Frequency:
            UpdateDDC1Frequency(Channel);
            break;

        default:
            break;
    }

    //forwatd event to RXPanel
    m_RXPanels[Channel]->ReceiverCallback_ChannelEvent(Event);
}


void TMainWindow::Initialize(void)
{
 uint32_t i;
 int RBW;
 wxString s;
 double RBWs[RBW_COUNT];


    m_Receiver->GetIFRBWs(RBWs);

    for(i=0;i<RBW_COUNT;i++)
    {
        RBW=(int)(RBWs[i]+0.5);

        if(RBW>1000)
        {
            RBW+=50;
            RBW=(RBW/100)*100;
        }

        s=wxT(" ")+FormatFrequency(RBW);

        m_CBIFRBW->Append(s);
        m_CBSwpRBW->Append(s);
    }

    m_CBSwpRBW->SetSelection(m_CBSwpRBW->GetCount()-2);

    m_CBAttenuator->Append(wxT(" Off"));
    m_CBAttenuator->Append(wxT(" 6 dB"));
    m_CBAttenuator->Append(wxT(" 12 dB"));
    m_CBAttenuator->Append(wxT(" 18 dB"));

    m_IFSpectrum->SetRange(m_DeviceInfo.FrontEndWindowWidth);
    m_IFSpectrum->SetBand(SpectrumBand_BothSides,true);

    UpdateAttenuatorState();
    UpdatePreamplifierState();
    UpdateFrontEndFrequency();
    UpdateIFRBWIndex();

    m_BSpectrum->SetValue(true);

    m_BSwpLoop->SetValue(m_SwpLoop);

    for(i=0;i<m_DeviceInfo.ChannelCount;i++)
    {
        m_RXPanels[i]->Initialize();
    }


    UpdateState();

    UpdateActiveChannel();
}

void TMainWindow::OnRXButtonClick(wxCommandEvent &Event)
{
 uint32_t Channel;

    Channel=Event.GetId()-ID_RX_BASE;

    if(Channel==m_Receiver->GetActiveChannel())
    {
        m_BRX[Channel]->SetValue(true);
    }
    else
    {
        m_Receiver->SetActiveChannel(Channel);
    }
}

void TMainWindow::OnButtonClick(wxCommandEvent &Event)
{
 wxBusyCursor *BusyCursor;

    switch(Event.GetId())
    {
        case ID_POWER:
            BusyCursor=new wxBusyCursor;

            m_BPower->Update();

            m_Receiver->SetPower(m_BPower->GetValue());

            delete BusyCursor;
            break;

        case ID_PREAMP:
            m_Receiver->SetPreamplifier(m_BPreamp->GetValue());
            break;

        case ID_IF_DOWN:
            m_Receiver->FrontEndFrequencyDown();
            break;

        case ID_IF_UP:
            m_Receiver->FrontEndFrequencyUp();
            break;

        case ID_SPECTRUM:
            if(!m_BSpectrum->GetValue())
            {
                m_BSpectrum->SetValue(true);
                return;
            }
            m_BSweeper->SetValue(false);
            UpdateBottomPanel();
            break;

        case ID_SWEEPER:
            if(!m_BSweeper->GetValue())
            {
                m_BSweeper->SetValue(true);
                return;
            }
            m_BSpectrum->SetValue(false);
            UpdateBottomPanel();
            break;

        case ID_SWP_START:
            if(m_BSwpStart->GetValue())
            {
                m_Receiver->StartSweeper(m_ESwpStart->Get(),m_ESwpEnd->Get(),m_CBSwpRBW->GetSelection());
            }
            break;

        case ID_SWP_STOP:
            m_Receiver->StopSweeper();
            break;

        case ID_SWP_CLEAR:
            m_SwpSpectrum->Clear();
            break;

        case ID_SWP_LOOP:
            m_SwpLoop=m_BSwpLoop->GetValue();
            break;
    }
}

void TMainWindow::OnComboBoxChanged(wxCommandEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_ATTENUATOR:
            m_Receiver->SetAttenuator(m_CBAttenuator->GetSelection());
            break;

        case ID_IFRBW:
            m_Receiver->SetIFRBWIndex(m_CBIFRBW->GetSelection());
            break;
    }
}

void TMainWindow::UpdateState(void)
{
 bool Power;
 bool Enabled;
 bool Connected;
 uint32_t i;

    Power=m_Receiver->GetPower();
    Connected=m_Receiver->IsConnected();

    if(Power && Connected)
    {
        m_PowerStateTimer.Start(500);
    }
    else
    {
        m_PowerStateTimer.Stop();
    }

    Enabled=Power && Connected && !m_Receiver->GetSweeper();

    m_BSpectrum->Enable(!m_Receiver->GetSweeper());

    m_BPower->SetValue(Power);
    m_IFSpectrum->Enable(Enabled);
    m_IFSpectrum->Clear();

    m_LRBW->Enable(Enabled);
    m_CBIFRBW->Enable(Enabled);

    m_BIFUp->Enable(Enabled);
    m_BIFDown->Enable(Enabled);

    UpdateAttenuatorState();
    UpdatePreamplifierState();

    for(i=0;i<m_DeviceInfo.ChannelCount;i++)
    {
        m_RXPanels[i]->UpdateState();
    }

    UpdateSwpState();
}



void TMainWindow::UpdateAttenuatorState(void)
{
 bool Enabled;

    m_CBAttenuator->SetSelection(m_Receiver->GetAttenuator());

    Enabled=m_Receiver->GetPower() && m_Receiver->GetHF() && !m_Receiver->GetSweeper();

    m_CBAttenuator->Enable(Enabled);
    m_LAttenuator->Enable(Enabled);
}

void TMainWindow::UpdatePreamplifierState(void)
{
    m_BPreamp->SetValue(m_Receiver->GetPreamplifier());

    m_BPreamp->Enable(m_Receiver->GetPower() && !m_Receiver->GetHF() && !m_Receiver->GetSweeper());
}

void TMainWindow::ReceiverCallback_IFSpectrum(float *Data,uint32_t Count,double Range,double Offset)
{
    m_IFSpectrum->SetData(Data,Count,Range,Offset);
}

void TMainWindow::ReceiverCallbask_Sweeper(float *Data,uint32_t Count)
{
    m_SwpSpectrum->AddData(Data,Count);
}

void TMainWindow::ReceiverCallback_DDC1Spectrum(uint32_t Channel,float *Data,uint32_t Count,double Range,double Offset)
{
    m_RXPanels[Channel]->ReceiverCallback_DDC1Spectrum(Data,Count,Range,Offset);
}

void TMainWindow::ReceiverCallback_DDC2Spectrum(uint32_t Channel,float *Data,uint32_t Count,double Range,double Offset)
{
    m_RXPanels[Channel]->ReceiverCallback_DDC2Spectrum(Data,Count,Range,Offset);
}

void TMainWindow::ReceiverCallback_AudioSpectrum(uint32_t Channel,float *Data,uint32_t Count,uint32_t Range)
{
    m_RXPanels[Channel]->ReceiverCallback_AudioSpectrum(Data,Count,Range);
}

void TMainWindow::UpdateFrontEndFrequency(void)
{
 int64_t Frequency;

    Frequency=m_Receiver->GetFrontEndFrequency();

    m_IFSpectrum->SetCenterFrequency(Frequency);
}

void TMainWindow::UpdateIFRBWIndex(void)
{
    m_CBIFRBW->SetSelection(m_Receiver->GetIFRBWIndex());
}

void TMainWindow::ReceiverCallback_Audio(uint32_t Channel,const float *Samples,uint32_t Count)
{
    m_RXPanels[Channel]->ReceiverCallback_Audio(Samples,Count);
}

void TMainWindow::UpdateActiveChannel(void)
{
 uint32_t Channel,i;
 TRXPanel *Panel;

    Channel=m_Receiver->GetActiveChannel();

    if(m_ActiveRXPanel!=m_RXPanels[Channel])
    {
        for(i=0;i<m_DeviceInfo.ChannelCount;i++)
        {
            m_BRX[i]->SetValue(i==Channel);
        }

        Panel=m_ActiveRXPanel;

        m_ActiveRXPanel=m_RXPanels[Channel];
        m_ActiveRXPanel->Show(true);

        if(Panel)
        {
            Panel->Show(false);
        }

        UpdateDDC1Frequency(Channel);
        UpdateDDC1(Channel);

        Layout();
    }
}

void TMainWindow::UpdateDDC1Frequency(uint32_t Channel)
{
    if(m_Receiver->GetActiveChannel()==Channel)
    {
        m_IFSpectrum->SetBandFrequency(m_Receiver->GetDDC1Frequency(Channel));
    }
}

void TMainWindow::UpdateDDC1(uint32_t Channel)
{
    if(m_Receiver->GetActiveChannel()==Channel)
    {
        m_IFSpectrum->SetBandWidth(m_Receiver->GetDDC1Info(Channel).Bandwidth);
    }
}

void TMainWindow::OnIFSpectrumTune(TSpectrumTuneEvent &Event)
{
    m_Receiver->SetDDC1Frequency(m_Receiver->GetActiveChannel(),(int32_t)Event.GetFrequency());
}

void TMainWindow::OnSwpSpectrumTune(TSpectrumTuneEvent &Event)
{
    m_Receiver->SetDemAbsFrequency(m_Receiver->GetActiveChannel(),Event.GetFrequency());
}

void TMainWindow::UpdateHFUHF(void)
{
    UpdateAttenuatorState();
    UpdatePreamplifierState();
}

void TMainWindow::OnTimer(wxTimerEvent& Event)
{
 wxBusyCursor *BusyCursor;

    switch(Event.GetId())
    {
        case ID_POWER_STATE_TIMER:
            if(!m_Receiver->GetPowerDirect())
            {
                wxMessageBox(wxT("WARNING: The receiver's temperature is too high.\n")
                             wxT("The receiver has been turned off to prevent damage.\n\n")
                             wxT("Please wait a moment and then turn the receiver on again.\n")
                             wxT("If the problem persists, please ensure that the receiver's fan\n")
                             wxT("and free air flow around the receiver are not obstructed."),wxT("Warning"),wxICON_WARNING|wxOK);
            }
            break;

        case ID_CONN_TIMER:
            if(!m_Receiver->IsConnected())
            {
                m_ConnTimer.Stop();

                UpdateState();
                wxMessageBox(APP_TITLE wxT(" will be closed.\nReceiver is no longer connected."),wxT("Warning"),wxICON_WARNING|wxOK);
                Close();
            }
            break;

        case ID_INIT_POWER_ON_TIMER:
            BusyCursor=new wxBusyCursor;

            m_BPower->SetValue(true);
            m_BPower->Update();

            m_Receiver->SetPower(true);

            delete BusyCursor;
            break;
    }
}

void TMainWindow::UpdateBottomPanel(void)
{
 bool IFSpectrum;
 bool Sweeper;

    IFSpectrum=m_BSpectrum->GetValue();
    Sweeper=!IFSpectrum;

    if(IFSpectrum)
    {
        m_IFSpectrum->Show(true);
        m_SwpSpectrum->Show(false);
    }
    else
    {
        m_SwpSpectrum->Show(true);
        m_IFSpectrum->Show(false);
    }

    m_CBIFRBW->Show(IFSpectrum);
    m_BIFDown->Show(IFSpectrum);
    m_BIFUp->Show(IFSpectrum);

    m_CBSwpRBW->Show(Sweeper);
    m_LSwpStart->Show(Sweeper);
    m_ESwpStart->Show(Sweeper);
    m_LSwpEnd->Show(Sweeper);
    m_ESwpEnd->Show(Sweeper);
    m_BSwpStart->Show(Sweeper);
    m_BSwpStop->Show(Sweeper);
    m_BSwpLoop->Show(Sweeper);
    m_BSwpClear->Show(Sweeper);

    Layout();
}

void TMainWindow::OnNumericEditChanged(wxNotifyEvent &Event)
{
 int64_t Freq;
 int64_t Max;

    switch(Event.GetId())
    {
        case ID_SWP_START_FREQ:
            Freq=((m_ESwpStart->Get()+SWP_STEP/2)/SWP_STEP)*SWP_STEP;

            Max=m_DeviceInfo.FrontEndMaxFrequency+m_DeviceInfo.FrontEndWindowWidth/2-SWP_STEP;

            if(Freq>Max)
            {
                Freq=Max;
            }

            if(Freq!=m_ESwpStart->Get())
            {
                m_ESwpStart->Set(Freq);
            }

            if(m_ESwpEnd->Get()<=Freq)
            {
                m_ESwpEnd->Set(Freq+SWP_STEP);
            }
            break;

        case ID_SWP_END_FREQ:
            Freq=((m_ESwpEnd->Get()+SWP_STEP/2)/SWP_STEP)*SWP_STEP;

            if(Freq<SWP_STEP)
            {
                Freq=SWP_STEP;
            }

            if(Freq!=m_ESwpEnd->Get())
            {
                m_ESwpEnd->Set(Freq);
            }

            if(m_ESwpStart->Get()>=Freq)
            {
                m_ESwpStart->Set(Freq-SWP_STEP);
            }
            break;
    }
}

void TMainWindow::UpdateSwpState(void)
{
 bool Enabled;
 bool Sweeper;

    Enabled=m_Receiver->GetPower() && m_Receiver->IsConnected();
    Sweeper=m_Receiver->GetSweeper();

    m_SwpSpectrum->Enable(Enabled);

    m_LRBW->Enable(Enabled && !Sweeper);
    m_CBSwpRBW->Enable(Enabled && !Sweeper);
    m_LSwpStart->Enable(Enabled && !Sweeper);
    m_ESwpStart->Enable(Enabled && !Sweeper);
    m_LSwpEnd->Enable(Enabled && !Sweeper);
    m_ESwpEnd->Enable(Enabled && !Sweeper);

    m_BSwpStart->Enable(Enabled);
    m_BSwpStart->SetValue(Sweeper);

    m_BSwpStop->Enable(Enabled && Sweeper);


    m_BSwpLoop->Enable(Enabled);
    m_BSwpClear->Enable(Enabled && !Sweeper);
}

bool TMainWindow::ReceiverCallback_SweeperQueryEnd(void)
{
    if(!m_SwpLoop)
    {
        wxCommandEvent Event(wxEVT_COMMAND_STOP_SWEEPER);

        //queue event which will stop sweeping in main thread
        AddPendingEvent(Event);
    }

    return !m_SwpLoop;
}

void TMainWindow::OnStopSweeper(wxCommandEvent &Event)
{
    m_Receiver->StopSweeper();
}

