/*****************************************************************************/
/*                        WiNRADiO G39DDC EXCELSIOR                          */
/*                                SDK DEMO                                   */
/*                           Channel (RX) panel                              */
/*                        (C) Copyright RADIXON 2014                         */
/*****************************************************************************/

#include "u_rxpanel.h"
#include "u_global.h"
#include "u_frequencyedit.h"
#include <wx/richtext/richtextctrl.h>


enum
{
    ID_FREQUENCY=wxID_HIGHEST,
    ID_DDC1_SPECTRUM,
    ID_DDC2_SPECTRUM,
    ID_AUDIO_SPECTRUM,
    ID_DDC1BW,
    ID_DDC1RBW,
    ID_DDC2BW,
    ID_ASR,
    ID_DEMBW,
    ID_DDC2RBW,
    ID_AUDIORBW,
    ID_MEASUR_TIMER,
    ID_VOLUME,
    ID_EVOLUME,
    ID_MUTE,
    ID_AGC,
    ID_GAIN,
    ID_SQUELCH,
    ID_SQUELCH_LEVEL,
    ID_AUDIO_FILTER,
    ID_AUDIO_FILTER_PARAMS,
    ID_AMS_SIDEBAND,
    ID_DSB_SIDEBAND,
    ID_ISB_SIDEBAND,
    ID_CW_TONE,
    ID_FMW_STEREO,
    ID_FMW_DEEM_50,
    ID_FMW_DEEM_75,

    ID_MODE_BASE=(wxID_HIGHEST+110)
};

BEGIN_EVENT_TABLE(TRXPanel, wxPanel)
    EVT_COMBOBOX(ID_DDC1BW,TRXPanel::OnComboBoxChanged)
    EVT_COMBOBOX(ID_DDC1RBW,TRXPanel::OnComboBoxChanged)
    EVT_COMBOBOX(ID_DDC2BW,TRXPanel::OnComboBoxChanged)
    EVT_COMBOBOX(ID_DDC2RBW,TRXPanel::OnComboBoxChanged)
    EVT_COMBOBOX(ID_ASR,TRXPanel::OnComboBoxChanged)
    EVT_COMBOBOX(ID_AUDIORBW,TRXPanel::OnComboBoxChanged)
    EVT_TIMER(ID_MEASUR_TIMER,TRXPanel::OnTimer)
    EVT_NUMERICEDIT(ID_FREQUENCY,TRXPanel::OnNumericEditChanged)
    EVT_NUMERICEDIT(ID_DEMBW,TRXPanel::OnNumericEditChanged)
    EVT_SPECTRUM_TUNE(ID_DDC1_SPECTRUM,TRXPanel::OnSpectrumTune)
    EVT_SPECTRUM_TUNE(ID_DDC2_SPECTRUM,TRXPanel::OnSpectrumTune)
    EVT_SPECTRUM_TUNE(ID_AUDIO_SPECTRUM,TRXPanel::OnSpectrumTune)
    EVT_NUMERICEDIT(ID_EVOLUME,TRXPanel::OnNumericEditChanged)
    EVT_COMMAND_SCROLL(ID_VOLUME,TRXPanel::OnSliderChanged)
    EVT_TOGGLEBUTTON(ID_MUTE,TRXPanel::OnButtonClick)
    EVT_COMBOBOX(ID_AGC,TRXPanel::OnComboBoxChanged)
    EVT_NUMERICEDIT(ID_GAIN,TRXPanel::OnNumericEditChanged)
    EVT_TOGGLEBUTTON(ID_AUDIO_FILTER,TRXPanel::OnButtonClick)
    EVT_NUMERICEDIT(ID_AUDIO_FILTER_PARAMS,TRXPanel::OnNumericEditChanged)
    EVT_COMBOBOX(ID_AMS_SIDEBAND,TRXPanel::OnComboBoxChanged)
    EVT_COMBOBOX(ID_DSB_SIDEBAND,TRXPanel::OnComboBoxChanged)
    EVT_COMBOBOX(ID_ISB_SIDEBAND,TRXPanel::OnComboBoxChanged)
    EVT_NUMERICEDIT(ID_CW_TONE,TRXPanel::OnNumericEditChanged)
    EVT_TOGGLEBUTTON(ID_FMW_STEREO,TRXPanel::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_FMW_DEEM_50,TRXPanel::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_FMW_DEEM_75,TRXPanel::OnButtonClick)
END_EVENT_TABLE()

TRXPanel::TRXPanel(wxWindow *Parent,TReceiver *Receiver,uint32_t Channel):wxPanel(Parent,wxID_ANY,wxPoint(0,0),wxSize(100,100),wxCLIP_CHILDREN|wxBORDER_NONE),
    m_MeasurTimer(this,ID_MEASUR_TIMER)
{
 uint32_t i;
 wxString s;
 int x1,x2,y,h,w,r;

    m_Receiver=Receiver;
    m_Channel=Channel;

    m_DeviceInfo=Receiver->GetDeviceInfo();
    m_DemModeCount=Receiver->GetDemModeCount();

    m_MainSizer=new wxBoxSizer(wxVERTICAL);
    SetSizer(m_MainSizer);

    m_DDC1Sizer=new wxBoxSizer(wxVERTICAL);
    m_DDC1SizerTop=new wxBoxSizer(wxHORIZONTAL);

    m_DDC2AudioSizer=new wxBoxSizer(wxHORIZONTAL);
    m_DDC2Sizer=new wxBoxSizer(wxVERTICAL);
    m_DDC2SizerTop=new wxBoxSizer(wxHORIZONTAL);
    m_AudioSizer=new wxBoxSizer(wxVERTICAL);
    m_AudioSizerTop=new wxBoxSizer(wxHORIZONTAL);

    m_DDC2AudioSizer->Add(m_DDC2Sizer,1,wxEXPAND|wxRIGHT,2);
    m_DDC2AudioSizer->Add(m_AudioSizer,1,wxEXPAND|wxLEFT,2);

    m_DDC2Sizer->Add(m_DDC2SizerTop,0,wxEXPAND|wxBOTTOM,4);
    m_AudioSizer->Add(m_AudioSizerTop,0,wxEXPAND|wxBOTTOM,4);


    m_EFrequency=new TFrequencyEdit(this,ID_FREQUENCY,15000000,-10000000,3508000000,wxPoint(0,0),wxSize(150,23));


    m_BMode=new wxToggleButton*[m_DemModeCount];

    for(i=0;i<m_DemModeCount;i++)
    {
        m_BMode[i]=new wxToggleButton(this,ID_MODE_BASE+i,m_Receiver->GetDemModeName(i),wxPoint(m_EFrequency->GetRect().GetRight()+4+i*76,0),wxSize(72,23));
    }

    Connect(ID_MODE_BASE,ID_MODE_BASE+m_DemModeCount-1,wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,wxCommandEventHandler(TRXPanel::OnModeButtonClick));

    m_BoxAMSSideband=new wxStaticBox(this,wxID_ANY,wxT("AMS sideband"),wxPoint(m_BMode[0]->GetRect().GetLeft(),m_BMode[0]->GetRect().GetBottom()+1),wxSize(85,51));
    m_CBAMSSideband=new wxComboBox(this,ID_AMS_SIDEBAND,wxEmptyString,wxPoint(m_BoxAMSSideband->GetRect().GetLeft()+8,m_BoxAMSSideband->GetRect().GetTop()+18),wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);

    m_BoxDSBSideband=new wxStaticBox(this,wxID_ANY,wxT("DSB sideband"),wxPoint(m_BoxAMSSideband->GetRect().GetRight()+4,m_BoxAMSSideband->GetRect().GetTop()),wxSize(85,51));
    m_CBDSBSideband=new wxComboBox(this,ID_DSB_SIDEBAND,wxEmptyString,wxPoint(m_BoxDSBSideband->GetRect().GetLeft()+8,m_BoxDSBSideband->GetRect().GetTop()+18),wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);

    m_BoxISBSideband=new wxStaticBox(this,wxID_ANY,wxT("ISB sideband"),wxPoint(m_BoxDSBSideband->GetRect().GetRight()+4,m_BoxDSBSideband->GetRect().GetTop()),wxSize(80,51));
    m_CBISBSideband=new wxComboBox(this,ID_ISB_SIDEBAND,wxEmptyString,wxPoint(m_BoxISBSideband->GetRect().GetLeft()+8,m_BoxISBSideband->GetRect().GetTop()+18),wxSize(65,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);

    m_BoxCWTone=new wxStaticBox(this,wxID_ANY,wxT("CW tone"),wxPoint(m_BoxISBSideband->GetRect().GetRight()+4,m_BoxISBSideband->GetRect().GetTop()),wxSize(70,51));
    m_ECWTone=new TFrequencyEdit(this,ID_CW_TONE,800,1,20000,wxPoint(m_BoxCWTone->GetRect().GetLeft()+6,m_BoxCWTone->GetRect().GetTop()+18),wxSize(59,23));
    m_ECWTone->SetUnits(FrequencyEditUnits_Hz);

    m_BoxFMW=new wxStaticBox(this,wxID_ANY,wxT("FMW"),wxPoint(m_BoxCWTone->GetRect().GetRight()+4,m_BoxCWTone->GetRect().GetTop()),wxSize(169,51));
    m_BFMWStereo=new wxToggleButton(this,ID_FMW_STEREO,wxT("Stereo"),wxPoint(m_BoxFMW->GetRect().GetLeft()+5,m_BoxFMW->GetRect().GetTop()+18),wxSize(43,23));
    m_LFMWDeem=new wxStaticText(this,wxID_ANY,wxT("De-em"),wxPoint(m_BFMWStereo->GetRect().GetRight()+8,m_BFMWStereo->GetRect().GetTop()));
    m_LFMWDeem->SetPosition(wxPoint(m_LFMWDeem->GetPosition().x,m_LFMWDeem->GetPosition().y+(23-m_LFMWDeem->GetRect().GetHeight())/2));
    m_BFMWDeem50=new wxToggleButton(this,ID_FMW_DEEM_50,wxT("50 us"),wxPoint(m_LFMWDeem->GetRect().GetRight()+4,m_BFMWStereo->GetRect().GetTop()),wxSize(36,23));
    m_BFMWDeem75=new wxToggleButton(this,ID_FMW_DEEM_75,wxT("75 us"),wxPoint(m_BFMWDeem50->GetRect().GetRight()+2,m_BFMWDeem50->GetRect().GetTop()),wxSize(36,23));

    m_BoxAF=new wxStaticBox(this,wxID_ANY,wxT("Audio filter"),wxPoint(m_BoxFMW->GetRect().GetRight()+4,m_BoxFMW->GetRect().GetTop()),wxSize(175,102));
    m_BAF=new wxToggleButton(this,ID_AUDIO_FILTER,wxT("Enabled"),wxPoint(m_BoxAF->GetRect().GetLeft()+8,m_BoxAF->GetRect().GetTop()+18),wxSize(50,23));
    m_LAFLow=new wxStaticText(this,wxID_ANY,wxT("Low"),wxPoint(m_BoxAF->GetRect().GetLeft()+65,m_BoxAF->GetRect().GetTop()+18));
    m_EAFLow=new TFrequencyEdit(this,ID_AUDIO_FILTER_PARAMS,300,0,64000,wxPoint(m_BoxAF->GetRect().GetLeft()+100,m_LAFLow->GetRect().GetTop()),wxSize(65,23));
    m_EAFLow->SetUnits(FrequencyEditUnits_kHz);
    m_LAFLow->SetPosition(wxPoint(m_LAFLow->GetPosition().x,m_LAFLow->GetPosition().y+(23-m_LAFLow->GetRect().GetHeight())/2));

    m_LAFHigh=new wxStaticText(this,wxID_ANY,wxT("High"),wxPoint(m_LAFLow->GetRect().GetLeft(),m_EAFLow->GetRect().GetBottom()+2));
    m_EAFHigh=new TFrequencyEdit(this,ID_AUDIO_FILTER_PARAMS,3000,1,64000,wxPoint(m_BoxAF->GetRect().GetLeft()+100,m_LAFHigh->GetRect().GetTop()),wxSize(65,23));
    m_EAFHigh->SetUnits(FrequencyEditUnits_kHz);
    m_LAFHigh->SetPosition(wxPoint(m_LAFHigh->GetPosition().x,m_LAFHigh->GetPosition().y+(23-m_LAFHigh->GetRect().GetHeight())/2));

    m_LAFDeem=new wxStaticText(this,wxID_ANY,wxT("De-em"),wxPoint(m_LAFLow->GetRect().GetLeft(),m_EAFHigh->GetRect().GetBottom()+2));
    m_EAFDeem=new TFloatEdit(this,ID_AUDIO_FILTER_PARAMS,0,-99,0,1,wxPoint(m_BoxAF->GetRect().GetLeft()+100,m_LAFDeem->GetRect().GetTop()),wxSize(30,23));
    m_LAFDeemUnits=new wxStaticText(this,wxID_ANY,wxT("dB/oct"),wxPoint(m_EAFDeem->GetRect().GetRight()+4,m_EAFDeem->GetRect().GetTop()));
    m_LAFDeem->SetPosition(wxPoint(m_LAFDeem->GetPosition().x,m_LAFDeem->GetPosition().y+(23-m_LAFDeem->GetRect().GetHeight())/2));
    m_LAFDeemUnits->SetPosition(wxPoint(m_LAFDeemUnits->GetPosition().x,m_LAFDeemUnits->GetPosition().y+(23-m_LAFDeemUnits->GetRect().GetHeight())/2));

    m_BoxVolume=new wxStaticBox(this,wxID_ANY,wxT("Volume"),wxPoint(m_BoxAMSSideband->GetRect().GetLeft(),m_BoxAMSSideband->GetRect().GetBottom()+1),wxSize(173,51));

    m_Volume=new wxSlider(this,ID_VOLUME,0,0,31,wxPoint(m_BoxVolume->GetPosition().x+4,m_BoxVolume->GetPosition().y+18),wxSize(78,23));
    m_EVolume=new TIntEdit(this,ID_EVOLUME,0,0,31,wxPoint(m_Volume->GetRect().GetRight()+4,m_Volume->GetRect().GetTop()),wxSize(30,23));
    m_BMute=new wxToggleButton(this,ID_MUTE,wxT("Mute"),wxPoint(m_EVolume->GetRect().GetRight()+4,m_EVolume->GetRect().GetTop()),wxSize(45,23));


    m_BoxAGC=new wxStaticBox(this,wxID_ANY,wxT("AGC"),wxPoint(m_BoxVolume->GetRect().GetRight()+4,m_BoxVolume->GetRect().GetTop()),wxSize(85,51));
    m_CBAGC=new wxComboBox(this,ID_AGC,wxEmptyString,wxPoint(m_BoxAGC->GetRect().GetLeft()+8,m_BoxAGC->GetRect().GetTop()+18),wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);

    m_BoxGain=new wxStaticBox(this,wxID_ANY,wxT("Gain"),wxPoint(m_BoxAGC->GetRect().GetRight()+4,m_BoxAGC->GetRect().GetTop()),wxSize(85,51));
    m_EGain=new TIntEdit(this,ID_GAIN,0,0,100,wxPoint(m_BoxGain->GetRect().GetLeft()+8,m_BoxGain->GetRect().GetTop()+18),wxSize(50,23));
    m_LGaindB=new wxStaticText(this,wxID_ANY,wxT("dB"),wxPoint(m_EGain->GetRect().GetRight()+4,m_EGain->GetRect().GetTop()));
    m_LGaindB->SetPosition(wxPoint(m_LGaindB->GetPosition().x,m_LGaindB->GetPosition().y+23/2-m_LGaindB->GetRect().GetHeight()/2));

    m_Squelched=false;
    m_BoxSquelch=new wxStaticBox(this,wxID_ANY,wxT("Squelch"),wxPoint(m_BoxGain->GetRect().GetRight()+4,m_BoxGain->GetRect().GetTop()),wxSize(149,51));
    m_BSquelch=new wxToggleButton(this,ID_SQUELCH,wxT("Enabled"),wxPoint(m_BoxSquelch->GetRect().GetLeft()+8,m_BoxSquelch->GetRect().GetTop()+18),wxSize(55,23));
    m_ESquelch=new TIntEdit(this,ID_SQUELCH_LEVEL,-50,-140,0,wxPoint(m_BSquelch->GetRect().GetRight()+8,m_BSquelch->GetRect().GetTop()),wxSize(40,23));
    m_LSquelchdB=new wxStaticText(this,wxID_ANY,wxT("dB"),wxPoint(m_ESquelch->GetRect().GetRight()+4,m_ESquelch->GetRect().GetTop()));
    m_LSquelchdB->SetPosition(wxPoint(m_LSquelchdB->GetPosition().x,m_LSquelchdB->GetPosition().y+23/2-m_LSquelchdB->GetRect().GetHeight()/2));

    m_BoxMeasur=new wxStaticBox(this,wxID_ANY,wxT("Measurements"),wxPoint(m_EFrequency->GetRect().GetLeft(),m_EFrequency->GetRect().GetBottom()+1),wxSize(150,102));


    x1=m_BoxMeasur->GetRect().GetLeft()+8;
    y=m_BoxMeasur->GetRect().GetTop()+14;
    w=60;
    x2=m_BoxMeasur->GetRect().GetRight()-w-8;

    m_LSlevel=new wxStaticText(this,wxID_ANY,wxT("Signal level"),wxPoint(x1,y));

    h=m_LSlevel->GetRect().GetHeight();
    r=h;

    m_LSlevelValue=new wxStaticText(this,wxID_ANY,wxEmptyString,wxPoint(x2,y),wxSize(w,h),wxALIGN_RIGHT|wxST_NO_AUTORESIZE);
    y+=r;

    m_LADCLevel=new wxStaticText(this,wxID_ANY,wxT("ADC level"),wxPoint(x1,y));
    m_LADCLevelValue=new wxStaticText(this,wxID_ANY,wxEmptyString,wxPoint(x2,y),wxSize(w,h),wxALIGN_RIGHT|wxST_NO_AUTORESIZE);
    y+=r;

    m_LAMDepth=new wxStaticText(this,wxID_ANY,wxT("AM depth"),wxPoint(x1,y));
    m_LAMDepthValue=new wxStaticText(this,wxID_ANY,wxEmptyString,wxPoint(x2,y),wxSize(w,h),wxALIGN_RIGHT|wxST_NO_AUTORESIZE);
    y+=r;

    m_LFreqError=new wxStaticText(this,wxID_ANY,wxT("Freq. error"),wxPoint(x1,y));
    m_LFreqErrorValue=new wxStaticText(this,wxID_ANY,wxEmptyString,wxPoint(x2,y),wxSize(w,h),wxALIGN_RIGHT|wxST_NO_AUTORESIZE);
    y+=r;

    m_LDeviation=new wxStaticText(this,wxID_ANY,wxT("Deviation"),wxPoint(x1,y));
    m_LDeviationValue=new wxStaticText(this,wxID_ANY,wxEmptyString,wxPoint(x2,y),wxSize(w,h),wxALIGN_RIGHT|wxST_NO_AUTORESIZE);
    y+=r;

    m_LCurrentGain=new wxStaticText(this,wxID_ANY,wxT("Current gain"),wxPoint(x1,y));
    m_LCurrentGainValue=new wxStaticText(this,wxID_ANY,wxEmptyString,wxPoint(x2,y),wxSize(w,h),wxALIGN_RIGHT|wxST_NO_AUTORESIZE);


    m_MainSizer->AddSpacer(130);

    //DDC2 controls
    m_MainSizer->Add(m_DDC2AudioSizer,1,wxEXPAND);

    m_LDDC2BW=new wxStaticText(this,wxID_ANY,wxT("DDC2 BW"),wxDefaultPosition,wxDefaultSize,wxALIGN_RIGHT);
    m_DDC2SizerTop->Add(m_LDDC2BW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC2SizerTop->AddSpacer(4);

    m_CBDDC2BW=new wxComboBox(this,ID_DDC2BW,wxEmptyString,wxDefaultPosition,wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_DDC2SizerTop->Add(m_CBDDC2BW,0,wxALIGN_LEFT);

    m_DDC2SizerTop->AddSpacer(8);

    m_LDEMBW=new wxStaticText(this,wxID_ANY,wxT("DEM BW"));
    m_DDC2SizerTop->Add(m_LDEMBW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC2SizerTop->AddSpacer(4);

    m_EDEMBW=new TFrequencyEdit(this,ID_DEMBW,20,1,320000,wxDefaultPosition,wxSize(75,23));
    m_EDEMBW->SetUnits(FrequencyEditUnits_kHz);
    m_DDC2SizerTop->Add(m_EDEMBW,0,wxALIGN_LEFT);

    m_DDC2SizerTop->AddSpacer(8);

    m_LDDC2RBW=new wxStaticText(this,wxID_ANY,wxT("RBW"));
    m_DDC2SizerTop->Add(m_LDDC2RBW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC2SizerTop->AddSpacer(4);

    m_CBDDC2RBW=new wxComboBox(this,ID_DDC2RBW,wxEmptyString,wxDefaultPosition,wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_DDC2SizerTop->Add(m_CBDDC2RBW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC2SizerTop->AddSpacer(8);

    m_LLocked=new wxStaticText(this,wxID_ANY,wxEmptyString,wxDefaultPosition,wxSize(50,m_LDDC2RBW->GetRect().GetHeight()),wxST_NO_AUTORESIZE);
    m_DDC2SizerTop->Add(m_LLocked,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC2Spectrum=new TSpectrum(this,ID_DDC2_SPECTRUM);
    m_DDC2Spectrum->SetMinLevel(-160);
    m_DDC2Spectrum->SetBand(SpectrumBand_BothSides,true);
    s.sprintf(wxT("DDC2 spectrum / Channel %u"),m_Channel);
    m_DDC2Spectrum->SetText(s);
    m_DDC2Sizer->Add(m_DDC2Spectrum,1,wxEXPAND);


    //audio controls

    m_LASR=new wxStaticText(this,wxID_ANY,wxT("Audio sample rate"));
    m_AudioSizerTop->Add(m_LASR,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_AudioSizerTop->AddSpacer(4);

    m_CBASR=new wxComboBox(this,ID_ASR,wxEmptyString,wxDefaultPosition,wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_AudioSizerTop->Add(m_CBASR,0,wxALIGN_LEFT);

    m_AudioSizerTop->AddSpacer(8);

    m_LAudioRBW=new wxStaticText(this,wxID_ANY,wxT("RBW"));
    m_AudioSizerTop->Add(m_LAudioRBW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_AudioSizerTop->AddSpacer(4);

    m_CBAudioRBW=new wxComboBox(this,ID_AUDIORBW,wxEmptyString,wxDefaultPosition,wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_AudioSizerTop->Add(m_CBAudioRBW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_AudioSizerTop->AddSpacer(8);

    m_LSquelched=new wxStaticText(this,wxID_ANY,wxEmptyString,wxDefaultPosition,wxSize(60,m_LAudioRBW->GetRect().GetHeight()),wxST_NO_AUTORESIZE);
    m_AudioSizerTop->Add(m_LSquelched,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_AudioSizerTop->AddSpacer(8);

    m_LStereo=new wxStaticText(this,wxID_ANY,wxEmptyString,wxDefaultPosition,wxSize(50,m_LSquelched->GetRect().GetHeight()),wxST_NO_AUTORESIZE);
    m_AudioSizerTop->Add(m_LStereo,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_AudioSpectrum=new TSpectrum(this,ID_AUDIO_SPECTRUM);
    m_AudioSpectrum->SetMinLevel(-100);
    m_AudioSpectrum->SetBand(SpectrumBand_None,false);
    s.sprintf(wxT("Audio spectrum / Channel %u"),m_Channel);
    m_AudioSpectrum->SetText(s);
    m_AudioSizer->Add(m_AudioSpectrum,1,wxEXPAND);


    //DDC1 controls
    m_MainSizer->Add(m_DDC1Sizer,1,wxEXPAND);
    m_DDC1Sizer->Add(m_DDC1SizerTop,0,wxEXPAND|wxTOP|wxBOTTOM,4);
    m_LDDC1BW=new wxStaticText(this,wxID_ANY,wxT("DDC1 BW"));
    m_DDC1SizerTop->Add(m_LDDC1BW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC1SizerTop->AddSpacer(4);

    m_CBDDC1BW=new wxComboBox(this,ID_DDC1BW,wxEmptyString,wxDefaultPosition,wxSize(75,23),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    m_DDC1SizerTop->Add(m_CBDDC1BW,0,wxALIGN_LEFT);

    m_DDC1SizerTop->AddSpacer(8);

    m_LDDC1RBW=new wxStaticText(this,wxID_ANY,wxT("RBW"));
    m_DDC1SizerTop->Add(m_LDDC1RBW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC1SizerTop->AddSpacer(4);

    m_CBDDC1RBW=new wxComboBox(this,ID_DDC1RBW,wxEmptyString,wxDefaultPosition,wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_DDC1SizerTop->Add(m_CBDDC1RBW,0,wxALIGN_LEFT);

    m_DDC1Spectrum=new TSpectrum(this,ID_DDC1_SPECTRUM);
    m_DDC1Spectrum->SetMinLevel(-160);
    m_DDC1Spectrum->SetBand(SpectrumBand_BothSides,true);
    s.sprintf(wxT("DDC1 spectrum / Channel %u"),m_Channel);
    m_DDC1Spectrum->SetText(s);
    m_DDC1Sizer->Add(m_DDC1Spectrum,1,wxEXPAND);


    m_Locked=false;
    m_Stereo=false;

    SetSizer(m_MainSizer);


    Layout();
}

TRXPanel::~TRXPanel(void)
{
    delete[] m_BMode;
}

void TRXPanel::Initialize(void)
{
 uint32_t i,Count,SampleRate;
 G39DDC_DDC_INFO DDCInfo;
 const wxChar *Sidebands[3]={wxT(" Lower"),wxT(" Upper"),wxT(" Both")};

    Count=m_Receiver->GetDDC1Count(m_Channel);

    for(i=0;i<Count;i++)
    {
        m_Receiver->GetDDCInfo(i,&DDCInfo);

        m_CBDDC1BW->Append(wxT(" ")+FormatFrequency(DDCInfo.Bandwidth));
    }

    Count=m_Receiver->GetAudioSampleRateCount();

    for(i=0;i<Count;i++)
    {
        SampleRate=m_Receiver->GetAudioSampleRate(i);

        m_CBASR->Append(wxT(" ")+FormatFrequency(SampleRate));
    }

    m_CBAGC->Append(wxT(" Off"));
    m_CBAGC->Append(wxT(" Slow"));
    m_CBAGC->Append(wxT(" Medium"));
    m_CBAGC->Append(wxT(" Fast"));

    for(i=0;i<3;i++)
    {
        m_CBAMSSideband->Append(Sidebands[i]);
        m_CBDSBSideband->Append(Sidebands[i]);
        m_CBISBSideband->Append(Sidebands[i]);
    }

    UpdateDDC1();
    UpdateDDC1RBWList();
    UpdateDDC1AbsFrequency();

    UpdateDDC2List();
    UpdateDDC2();
    UpdateDDC2RBWList();
    UpdateDDC2Frequency();
    UpdateDDC2AbsFrequency();

    UpdateDemFrequency();
    UpdateDemAbsFrequency();
    UpdateDemBandwidth();
    UpdateDemMode();
    UpdateDemAMSSideband();
    UpdateDemDSBSideband();
    UpdateDemISBSideband();
    UpdateDemCWTone();
    UpdateDemFMWStereo();
    UpdateDemFMWDeemphasis();

    UpdateAudio();
    UpdateAudioRBWList();
    UpdateAudioVolume();
    UpdateAudioMute();
    UpdateAudioFilter();
    UpdateAudioFilterParams();

    UpdateAGC();
    UpdateGain();
}

void TRXPanel::UpdateDDC2List(void)
{
 uint32_t i,Count;
 G39DDC_DDC_INFO DDCInfo;

    m_CBDDC2BW->Clear();

    Count=m_Receiver->GetDDC2Count(m_Channel);

    for(i=0;i<Count;i++)
    {
        m_Receiver->GetDDCInfo(i,&DDCInfo);

        m_CBDDC2BW->Append(wxT(" ")+FormatFrequency(DDCInfo.Bandwidth));
    }
}


void TRXPanel::OnComboBoxChanged(wxCommandEvent &Event)
{
 static uint32_t SidebandIndexToSideband[3]={G39DDC_SIDE_BAND_LOWER,G39DDC_SIDE_BAND_UPPER,G39DDC_SIDE_BAND_BOTH};

    switch(Event.GetId())
    {
        case ID_DDC1BW:
            if(m_CBDDC1BW->GetSelection()>=0)
            {
                m_Receiver->SetDDC1(m_Channel,m_CBDDC1BW->GetSelection());
            }
            break;

        case ID_DDC1RBW:
            if(m_CBDDC1RBW->GetSelection()>=0)
            {
                m_Receiver->SetDDC1RBWIndex(m_Channel,m_CBDDC1RBW->GetSelection());
            }
            break;

        case ID_DDC2BW:
            if(m_CBDDC2BW->GetSelection()>=0)
            {
                m_Receiver->SetDDC2(m_Channel,m_CBDDC2BW->GetSelection());
            }
            break;

        case ID_DDC2RBW:
            if(m_CBDDC2RBW->GetSelection()>=0)
            {
                m_Receiver->SetDDC2RBWIndex(m_Channel,m_CBDDC2RBW->GetSelection());
            }
            break;

        case ID_ASR:
            if(m_CBASR->GetSelection()>=0)
            {
                m_Receiver->SetAudio(m_Channel,m_CBASR->GetSelection());
            }
            break;

        case ID_AUDIORBW:
            if(m_CBAudioRBW->GetSelection()>=0)
            {
                m_Receiver->SetAudioRBWIndex(m_Channel,m_CBAudioRBW->GetSelection());
            }
            break;

        case ID_AGC:
            if(m_CBAGC->GetSelection()>=0)
            {
                m_Receiver->SetAGC(m_Channel,m_CBAGC->GetSelection());
            }
            break;

        case ID_AMS_SIDEBAND:
            if(m_CBAMSSideband->GetSelection()>=0)
            {
                m_Receiver->SetDemAMSSideband(m_Channel,SidebandIndexToSideband[m_CBAMSSideband->GetSelection()]);
            }
            break;

        case ID_DSB_SIDEBAND:
            if(m_CBDSBSideband->GetSelection()>=0)
            {
                m_Receiver->SetDemDSBSideband(m_Channel,SidebandIndexToSideband[m_CBDSBSideband->GetSelection()]);
            }
            break;

        case ID_ISB_SIDEBAND:
            if(m_CBISBSideband->GetSelection()>=0)
            {
                m_Receiver->SetDemISBSideband(m_Channel,SidebandIndexToSideband[m_CBISBSideband->GetSelection()]);
            }
            break;
    }
}



void TRXPanel::OnButtonClick(wxCommandEvent &Event)
{
 uint32_t Deem;

    switch(Event.GetId())
    {
        case ID_MUTE:
            m_Receiver->SetMute(m_Channel,m_BMute->GetValue());
            break;

        case ID_AUDIO_FILTER:
            m_Receiver->SetAudioFilter(m_Channel,m_BAF->GetValue());
            break;

        case ID_FMW_STEREO:
            m_Receiver->SetDemFMWStereo(m_Channel,m_BFMWStereo->GetValue());
            break;

        case ID_FMW_DEEM_50:
            if(m_BFMWDeem50->GetValue())
            {
                m_BFMWDeem75->SetValue(false);
                Deem=50;
            }
            else
            {
                Deem=0;
            }
            m_Receiver->SetDemFMWDeemphasis(m_Channel,Deem);
            break;

        case ID_FMW_DEEM_75:
            if(m_BFMWDeem75->GetValue())
            {
                m_BFMWDeem50->SetValue(false);
                Deem=75;
            }
            else
            {
                Deem=0;
            }
            m_Receiver->SetDemFMWDeemphasis(m_Channel,Deem);
            break;
    }
}



void TRXPanel::ReceiverCallback_ChannelEvent(TChannelEvent Event)
{
    switch(Event)
    {
        case ChannelEvent_DDC1: //DDC1 type has been changed
            UpdateDDC1();
            break;

        case ChannelEvent_DDC1_RBWList:
            UpdateDDC1RBWList();
            break;

        case ChannelEvent_DDC1_RBWIndex:
            UpdateDDC1RBW();
            break;

        case ChannelEvent_DDC1_AbsFrequency:
            UpdateDDC1AbsFrequency();
            break;

        case ChannelEvent_DDC2:
            UpdateDDC2();
            break;

        case ChannelEvent_DDC2_List:
            UpdateDDC2List();
            break;

        case ChannelEvent_DDC2_RBWList:
            UpdateDDC2RBWList();
            break;

        case ChannelEvent_DDC2_RBWIndex:
            UpdateDDC2RBW();
            break;

        case ChannelEvent_DDC2_Frequency:
            UpdateDDC2Frequency();
            break;

        case ChannelEvent_DDC2_AbsFrequency:
            UpdateDDC2AbsFrequency();
            break;

        case ChannelEvent_DEM_Frequency:
            UpdateDemFrequency();
            break;

        case ChannelEvent_DEM_AbsFrequency:
            UpdateDemAbsFrequency();
            break;

        case ChannelEvent_DEM_Bandwidth:
            UpdateDemBandwidth();
            break;

        case ChannelEvent_DEM_Mode:
            UpdateDemMode();
            break;

        case ChannelEvent_DEM_AMSSideband:
            UpdateDemAMSSideband();
            break;

        case ChannelEvent_DEM_DSBSideband:
            UpdateDemDSBSideband();
            break;

        case ChannelEvent_DEM_ISBSideband:
            UpdateDemISBSideband();
            break;

        case ChannelEvent_DEM_CWTone:
            UpdateDemCWTone();
            break;

        case ChannelEvent_DEM_FMWStereo:
            UpdateDemFMWStereo();
            break;

        case ChannelEvent_DEM_FMWDeemphasis:
            UpdateDemFMWDeemphasis();
            break;

        case ChannelEvent_Audio:
            UpdateAudio();
            break;

        case ChannelEvent_Audio_RBWList:
            UpdateAudioRBWList();
            break;

        case ChannelEvent_Audio_RBWIndex:
            UpdateAudioRBW();
            break;

        case ChannelEvent_Audio_Volume:
            UpdateAudioVolume();
            break;

        case ChannelEvent_Audio_Mute:
            UpdateAudioMute();
            break;

        case ChannelEvent_Audio_Filter:
            UpdateAudioFilter();
            break;

        case ChannelEvent_Audio_FilterParams:
            UpdateAudioFilterParams();
            break;

        case ChannelEvent_AGC:
            UpdateAGC();
            break;

        case ChannelEvent_Gain:
            UpdateGain();
            break;

        default:
            break;
    }


}

void TRXPanel::UpdateDDC1(void)
{

    m_CBDDC1BW->SetSelection(m_Receiver->GetDDC1(m_Channel));

    m_DDC1Spectrum->SetRange(m_Receiver->GetDDC1Info(m_Channel).Bandwidth);
}

void TRXPanel::UpdateDDC2(void)
{
    m_CBDDC2BW->SetSelection(m_Receiver->GetDDC2(m_Channel));

    m_DDC2Spectrum->SetRange(m_Receiver->GetDDC2Info(m_Channel).Bandwidth);
    m_DDC1Spectrum->SetBandWidth(m_Receiver->GetDDC2Info(m_Channel).Bandwidth);
}

void TRXPanel::ReceiverCallback_DDC1Spectrum(float *Data,uint32_t Count,double Range,double Offset)
{
    m_DDC1Spectrum->SetData(Data,Count,Range,Offset);
}

void TRXPanel::ReceiverCallback_DDC2Spectrum(float *Data,uint32_t Count,double Range,double Offset)
{
    m_DDC2Spectrum->SetData(Data,Count,Range,Offset);
}

void TRXPanel::UpdateState(void)
{
 bool Power;
 bool Enabled;
 bool Connected;

    Power=m_Receiver->GetPower();
    Connected=m_Receiver->IsConnected();

    Enabled=Power && Connected && !m_Receiver->GetSweeper();

    if(Enabled)
    {
        m_MeasurTimer.Start(200);
    }
    else
    {
        m_MeasurTimer.Stop();
    }


    m_EFrequency->Enable(Enabled);

    m_BoxVolume->Enable(Enabled);
    m_Volume->Enable(Enabled);
    m_EVolume->Enable(Enabled);
    m_BMute->Enable(Enabled);

    m_BoxAGC->Enable(Enabled);
    m_CBAGC->Enable(Enabled);

    m_BoxSquelch->Enable(Enabled);
    m_BSquelch->Enable(Enabled);
    m_ESquelch->Enable(Enabled);
    m_LSquelchdB->Enable(Enabled);

    m_LDDC1BW->Enable(Enabled);
    m_CBDDC1BW->Enable(Enabled);
    m_LDDC1RBW->Enable(Enabled);
    m_CBDDC1RBW->Enable(Enabled);

    m_DDC1Spectrum->Enable(Enabled);

    m_DDC2Spectrum->Enable(Enabled);

    m_AudioSpectrum->Enable(Enabled);

    m_LDDC2BW->Enable(Enabled);
    m_CBDDC2BW->Enable(Enabled);

    m_LASR->Enable(Enabled);
    m_CBASR->Enable(Enabled);

    m_LDEMBW->Enable(Enabled);
    m_EDEMBW->Enable(Enabled);

    m_LDDC2RBW->Enable(Enabled);
    m_LAudioRBW->Enable(Enabled);
    m_CBDDC2RBW->Enable(Enabled);
    m_CBAudioRBW->Enable(Enabled);

    m_BoxAF->Enable(Enabled);
    m_BAF->Enable(Enabled);

    m_BoxMeasur->Enable(Enabled);
    m_LSlevel->Enable(Enabled);
    m_LSlevelValue->Enable(Enabled);
    m_LADCLevel->Enable(Enabled);
    m_LADCLevelValue->Enable(Enabled);
    m_LFreqError->Enable(Enabled);
    m_LFreqErrorValue->Enable(Enabled);

    m_LCurrentGain->Enable(Enabled);
    m_LCurrentGainValue->Enable(Enabled);

    if(!Enabled)
    {
        m_LADCLevelValue->SetLabel(wxEmptyString);
        m_LFreqErrorValue->SetLabel(wxEmptyString);
        m_LCurrentGainValue->SetLabel(wxEmptyString);
        m_LSlevelValue->SetLabel(wxEmptyString);
    }

    UpdateDemMode();
    UpdateAGC();
    UpdateAudioFilter();


    if(!Enabled)
    {
        m_DDC2Spectrum->Clear();
        m_AudioSpectrum->Clear();
        m_DDC1Spectrum->Clear();
    }
}

void TRXPanel::UpdateDDC1RBWList(void)
{
 int i;
 int RBW;
 double RBWs[RBW_COUNT];

    m_CBDDC1RBW->Clear();

    m_Receiver->GetDDC1RBWs(m_Channel,RBWs);

    for(i=0;i<RBW_COUNT;i++)
    {
        RBW=RBWs[i]+0.5;

        if(RBW>1000)
        {
            RBW+=50;
            RBW=(RBW/100)*100;
        }

        m_CBDDC1RBW->Append(wxT(" ")+FormatFrequency(RBW));
    }

    m_CBDDC1RBW->SetSelection(m_Receiver->GetDDC1RBWIndex(m_Channel));
}

void TRXPanel::UpdateDDC1RBW(void)
{
    m_CBDDC1RBW->SetSelection(m_Receiver->GetDDC1RBWIndex(m_Channel));
}

void TRXPanel::UpdateDDC2RBWList(void)
{
 int i;
 int RBW;
 double RBWs[RBW_COUNT];

    m_CBDDC2RBW->Clear();

    m_Receiver->GetDDC2RBWs(m_Channel,RBWs);

    for(i=0;i<RBW_COUNT;i++)
    {
        RBW=RBWs[i]+0.5;

        if(RBW>1000)
        {
            RBW+=50;
            RBW=(RBW/100)*100;
        }

        m_CBDDC2RBW->Append(wxT(" ")+FormatFrequency(RBW));
    }

    m_CBDDC2RBW->SetSelection(m_Receiver->GetDDC2RBWIndex(m_Channel));
}

void TRXPanel::UpdateDDC2RBW(void)
{
    m_CBDDC2RBW->SetSelection(m_Receiver->GetDDC2RBWIndex(m_Channel));
}

void TRXPanel::UpdateAudio(void)
{
 uint32_t SampleRate;

    m_CBASR->SetSelection(m_Receiver->GetAudio(m_Channel,&SampleRate));

    m_AudioSpectrum->SetRange(SampleRate/2);
    m_AudioSpectrum->SetCenterFrequency(SampleRate/4);

    m_WaveOut.Open(SampleRate);
}

void TRXPanel::UpdateAudioRBWList(void)
{
 int i;
 int RBW;
 double RBWs[RBW_COUNT];

    m_CBAudioRBW->Clear();

    m_Receiver->GetAudioRBWs(m_Channel,RBWs);

    for(i=0;i<RBW_COUNT;i++)
    {
        RBW=RBWs[i]+0.5;

        if(RBW>1000)
        {
            RBW+=50;
            RBW=(RBW/100)*100;
        }

        m_CBAudioRBW->Append(wxT(" ")+FormatFrequency(RBW));
    }

    m_CBAudioRBW->SetSelection(m_Receiver->GetAudioRBWIndex(m_Channel));
}

void TRXPanel::ReceiverCallback_AudioSpectrum(float *Data,uint32_t Count,uint32_t Range)
{
    m_AudioSpectrum->SetData(Data,Count,Range,0);
}

void TRXPanel::UpdateAudioRBW(void)
{
    m_CBAudioRBW->SetSelection(m_Receiver->GetAudioRBWIndex(m_Channel));
}

void TRXPanel::ReceiverCallback_Audio(const float *Samples,uint32_t Count)
{
    m_WaveOut.Write(Samples,Count);
}


void TRXPanel::OnTimer(wxTimerEvent& Event)
{
    switch(Event.GetId())
    {
        case ID_MEASUR_TIMER:
            ShowMeasurements();
            break;
    }
}

void TRXPanel::OnNumericEditChanged(wxNotifyEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_FREQUENCY:
            m_Receiver->SetDemAbsFrequency(m_Channel,m_EFrequency->Get());
            break;

        case ID_DEMBW:
            m_Receiver->SetDemBandwidth(m_Channel,m_EDEMBW->Get());
            break;

        case ID_EVOLUME:
            m_Receiver->SetVolume(m_Channel,(uint8_t)m_EVolume->Get());
            break;

        case ID_GAIN:
            m_Receiver->SetGain(m_Channel,(int32_t)m_EGain->Get());
            break;

        case ID_AUDIO_FILTER_PARAMS:
            m_Receiver->SetAudioFilterParams(m_Channel,m_EAFLow->Get(),m_EAFHigh->Get(),m_EAFDeem->Get()*0.1);
            break;

        case ID_CW_TONE:
            m_Receiver->SetDemCWTone(m_Channel,(uint32_t)m_ECWTone->Get());
            break;
    }
}

void TRXPanel::UpdateDDC1AbsFrequency(void)
{
    m_DDC1Spectrum->SetCenterFrequency(m_Receiver->GetDDC1AbsFrequency(m_Channel));
}

void TRXPanel::UpdateDDC2Frequency(void)
{
    m_DDC1Spectrum->SetBandFrequency(m_Receiver->GetDDC2Frequency(m_Channel));
}

void TRXPanel::UpdateDDC2AbsFrequency(void)
{
    m_DDC2Spectrum->SetCenterFrequency(m_Receiver->GetDDC2AbsFrequency(m_Channel));
}

void TRXPanel::UpdateDemFrequency(void)
{
    m_DDC2Spectrum->SetBandFrequency(m_Receiver->GetDemFrequency(m_Channel));
}

void TRXPanel::UpdateDemAbsFrequency(void)
{
    m_EFrequency->Set(m_Receiver->GetDemAbsFrequency(m_Channel));
}

void TRXPanel::UpdateDemBandwidth(void)
{
    m_DDC2Spectrum->SetBandWidth(m_Receiver->GetDemBandwidth(m_Channel));
    m_EDEMBW->Set(m_Receiver->GetDemBandwidth(m_Channel));
}

void TRXPanel::OnSpectrumTune(TSpectrumTuneEvent &Event)
{
 int32_t Low,High;

    switch(Event.GetId())
    {
        case ID_DDC1_SPECTRUM:
            m_Receiver->SetDDC2Frequency(m_Channel,(int32_t)Event.GetFrequency());
            break;

        case ID_DDC2_SPECTRUM:
            m_Receiver->SetDemFrequency(m_Channel,(int32_t)Event.GetFrequency());
            break;

        case ID_AUDIO_SPECTRUM:
            Low=(int32_t)Event.GetFrequency();

            High=Low+m_AudioSpectrum->GetBandWidth()/2;
            Low=High-m_AudioSpectrum->GetBandWidth();

            m_Receiver->SetAudioFilterParams(m_Channel,Low,High,m_EAFDeem->Get()*0.1);
            break;
    }
}

void TRXPanel::UpdateDemMode(void)
{
 uint32_t Mode;
 uint32_t i;
 bool Enabled;

    Enabled=m_Receiver->GetPower() && !m_Receiver->GetSweeper();

    Mode=m_Receiver->GetDemMode(m_Channel);

    for(i=0;i<m_DemModeCount;i++)
    {
        m_BMode[i]->SetValue(i==Mode);
        m_BMode[i]->Enable(Enabled);
    }

    switch(Mode)
    {
        case G39DDC_MODE_LSB:
            m_DDC2Spectrum->SetBand(SpectrumBand_LowerSide,true);
            break;

        case G39DDC_MODE_USB:
            m_DDC2Spectrum->SetBand(SpectrumBand_UpperSide,true);
            break;

        default:
            m_DDC2Spectrum->SetBand(SpectrumBand_BothSides,true);
            break;
    }

    m_BoxAMSSideband->Enable(Enabled && Mode==G39DDC_MODE_AMS);
    m_CBAMSSideband->Enable(Enabled && Mode==G39DDC_MODE_AMS);

    m_BoxDSBSideband->Enable(Enabled && Mode==G39DDC_MODE_DSB);
    m_CBDSBSideband->Enable(Enabled && Mode==G39DDC_MODE_DSB);

    m_BoxISBSideband->Enable(Enabled && Mode==G39DDC_MODE_ISB);
    m_CBISBSideband->Enable(Enabled && Mode==G39DDC_MODE_ISB);

    m_BoxCWTone->Enable(Enabled && Mode==G39DDC_MODE_CW);
    m_ECWTone->Enable(Enabled && Mode==G39DDC_MODE_CW);

    m_BoxFMW->Enable(Enabled && Mode==G39DDC_MODE_FMW);
    m_BFMWStereo->Enable(Enabled && Mode==G39DDC_MODE_FMW);
    m_BFMWDeem50->Enable(Enabled && Mode==G39DDC_MODE_FMW);
    m_BFMWDeem75->Enable(Enabled && Mode==G39DDC_MODE_FMW);

    if(Enabled && (Mode==G39DDC_MODE_AM || Mode==G39DDC_MODE_AMS))
    {
        m_LAMDepth->Enable(true);
        m_LAMDepthValue->Enable(true);
    }
    else
    {
        m_LAMDepth->Enable(false);
        m_LAMDepthValue->Enable(false);
        m_LAMDepthValue->SetLabel(wxEmptyString);
    }

    if(Enabled && (Mode==G39DDC_MODE_FM || Mode==G39DDC_MODE_FMW))
    {
        m_LDeviation->Enable(true);
        m_LDeviationValue->Enable(true);
    }
    else
    {
        m_LDeviation->Enable(false);
        m_LDeviationValue->Enable(false);
        m_LDeviationValue->SetLabel(wxEmptyString);
    }

    m_LLocked->SetLabel(wxEmptyString);
    m_Locked=false;

    m_LStereo->SetLabel(wxEmptyString);
    m_Stereo=false;
}

void TRXPanel::OnModeButtonClick(wxCommandEvent &Event)
{
 uint32_t Mode;

    Mode=Event.GetId()-ID_MODE_BASE;

    if(!m_BMode[Mode]->GetValue())
    {
        m_BMode[Mode]->SetValue(true);
        return;
    }

    m_Receiver->SetDemMode(m_Channel,Mode);
}

void TRXPanel::UpdateAudioVolume(void)
{
 uint8_t Volume;

    Volume=m_Receiver->GetVolume(m_Channel);

    m_Volume->SetValue(Volume);
    m_EVolume->Set(Volume);
}

void TRXPanel::OnSliderChanged(wxScrollEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_VOLUME:
            m_Receiver->SetVolume(m_Channel,(uint8_t)m_Volume->GetValue());
            break;
    }
}

void TRXPanel::UpdateAudioMute(void)
{
    m_BMute->SetValue(m_Receiver->GetMute(m_Channel));
}


void TRXPanel::UpdateAGC(void)
{
 uint32_t AGC;
 bool Enabled;

    AGC=m_Receiver->GetAGC(m_Channel);
    Enabled=m_Receiver->GetPower() && AGC==AGC_OFF && !m_Receiver->GetSweeper();

    m_CBAGC->SetSelection(AGC);

    m_BoxGain->Enable(Enabled);
    m_EGain->Enable(Enabled);
    m_LGaindB->Enable(Enabled);
}

void TRXPanel::UpdateGain(void)
{
    m_EGain->Set(m_Receiver->GetGain(m_Channel));
}

void TRXPanel::UpdateAudioFilter(void)
{
 bool AF;
 bool Enabled;

    AF=m_Receiver->GetAudioFilter(m_Channel);

    if(AF)
    {
        m_AudioSpectrum->SetBand(SpectrumBand_BothSides,false);
    }
    else
    {
        m_AudioSpectrum->SetBand(SpectrumBand_None,false);
    }

    Enabled=AF && m_Receiver->GetPower() && !m_Receiver->GetSweeper();

    m_LAFLow->Enable(Enabled);
    m_EAFLow->Enable(Enabled);
    m_LAFHigh->Enable(Enabled);
    m_EAFHigh->Enable(Enabled);
    m_LAFDeem->Enable(Enabled);
    m_EAFDeem->Enable(Enabled);
    m_LAFDeemUnits->Enable(Enabled);
}

void TRXPanel::UpdateAudioFilterParams(void)
{
 uint32_t Low,High;
 double Deem;

    m_Receiver->GetAudioFilterParams(m_Channel,&Low,&High,&Deem);

    m_EAFLow->Set(Low);
    m_EAFHigh->Set(High);
    m_EAFDeem->Set(Deem*10-0.5);

    m_AudioSpectrum->SetBandFrequency((High+Low)/2);
    m_AudioSpectrum->SetBandWidth(High-Low);
}

void TRXPanel::UpdateDemAMSSideband(void)
{
    switch(m_Receiver->GetDemAMSSideband(m_Channel))
    {
        case G39DDC_SIDE_BAND_LOWER:
            m_CBAMSSideband->SetSelection(0);
            break;

        case G39DDC_SIDE_BAND_UPPER:
            m_CBAMSSideband->SetSelection(1);
            break;

        default:
            m_CBAMSSideband->SetSelection(2);
            break;
    }
}

void TRXPanel::UpdateDemDSBSideband(void)
{
    switch(m_Receiver->GetDemDSBSideband(m_Channel))
    {
        case G39DDC_SIDE_BAND_LOWER:
            m_CBDSBSideband->SetSelection(0);
            break;

        case G39DDC_SIDE_BAND_UPPER:
            m_CBDSBSideband->SetSelection(1);
            break;

        default:
            m_CBDSBSideband->SetSelection(2);
            break;
    }
}

void TRXPanel::UpdateDemISBSideband(void)
{
    switch(m_Receiver->GetDemISBSideband(m_Channel))
    {
        case G39DDC_SIDE_BAND_LOWER:
            m_CBISBSideband->SetSelection(0);
            break;

        case G39DDC_SIDE_BAND_UPPER:
            m_CBISBSideband->SetSelection(1);
            break;

        default:
            m_CBISBSideband->SetSelection(2);
            break;
    }
}

void TRXPanel::UpdateDemCWTone(void)
{
    m_ECWTone->Set(m_Receiver->GetDemCWTone(m_Channel));
}

void TRXPanel::UpdateDemFMWStereo(void)
{
    m_BFMWStereo->SetValue(m_Receiver->GetDemFMWStereo(m_Channel));
}

void TRXPanel::UpdateDemFMWDeemphasis(void)
{
 uint32_t Deem;

    Deem=m_Receiver->GetDemFMWDeemphasis(m_Channel);

    m_BFMWDeem50->SetValue(Deem==50);
    m_BFMWDeem75->SetValue(Deem==75);
}

void TRXPanel::ShowMeasurements(void)
{
 float Level;
 bool Squelched;
 int32_t TuneError;
 wxString s;
 uint32_t ADCLevel;
 uint32_t Mode;
 bool Locked;
 bool Stereo;

    Mode=m_Receiver->GetDemMode(m_Channel);

    Level=m_Receiver->GetSignalLevel(m_Channel);

    Level=10.0*log10(Level*Level*(1000.0/50.0)); //convert V to dBm

    s.sprintf(wxT("%.0f dBm"),Level);
    m_LSlevelValue->SetLabel(s);

    if(m_BSquelch->GetValue())
    {
        Squelched=m_ESquelch->Get()>Level;
    }
    else
    {
        Squelched=false;
    }

    if(m_Squelched!=Squelched)
    {
        m_Squelched=Squelched;

        m_Receiver->SetSquelched(m_Channel,Squelched);
        m_LSquelched->SetLabel(Squelched?wxT("SQUELCHED"):wxEmptyString);
    }

    ADCLevel=m_Receiver->GetADCLevel();

    if(ADCLevel>=32767)
    {
        s=wxT("CLIP"); //clipping
    }
    else
    {
        s.sprintf(wxT("%d %%"),ADCLevel*100/32767);
    }

    m_LADCLevelValue->SetLabel(s);

    TuneError=m_Receiver->GetDemTuneError(m_Channel);

    s.sprintf(wxT("%.3f kHz"),TuneError*(1.0/1000.0));

    m_LFreqErrorValue->SetLabel(s);

    Locked=false;
    Stereo=false;

    switch(Mode)
    {
        case G39DDC_MODE_AMS:
            Locked=m_Receiver->IsDemAMSLocked(m_Channel);

        case G39DDC_MODE_AM:
            s.sprintf(wxT("%.0f %%"),m_Receiver->GetAMDepth(m_Channel));
            m_LAMDepthValue->SetLabel(s);
            break;

        case G39DDC_MODE_DSB:
            Locked=m_Receiver->IsDemDSBLocked(m_Channel);
            break;

        case G39DDC_MODE_FMW:
            Stereo=m_Receiver->IsDemFMWStereo(m_Channel);

        case G39DDC_MODE_FM:
            s.sprintf(wxT("%.3f kHz"),m_Receiver->GetDeviation(m_Channel)*(1.0/1000.0));
            m_LDeviationValue->SetLabel(s);
            break;
    }

    if(m_Locked!=Locked)
    {
        m_Locked=Locked;
        m_LLocked->SetLabel(Locked?wxT("LOCKED"):wxEmptyString);
    }

    if(m_Stereo!=Stereo)
    {
        m_Stereo=Stereo;
        m_LStereo->SetLabel(Stereo?wxT("STEREO"):wxEmptyString);
    }

    s.sprintf(wxT("%.1f dB"),m_Receiver->GetCurrentGain(m_Channel));
    m_LCurrentGainValue->SetLabel(s);
}


