/*****************************************************************************/
/*                        WiNRADiO G39DDC EXCELSIOR                          */
/*                                SDK DEMO                                   */
/*                       Simple sweeper spectrum panel                       */
/*                        (C) Copyright RADIXON 2014                         */
/*****************************************************************************/

#include "u_swpspectrum.h"
#include <wx/dcbuffer.h>
#include "u_global.h"

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(wxEVT_SWP_DATACHANGED,5003)
END_DECLARE_EVENT_TYPES()

DEFINE_EVENT_TYPE(wxEVT_SWP_DATACHANGED)

#define EVT_SWP_DATACHANGED(fn)                                 \
	DECLARE_EVENT_TABLE_ENTRY(wxEVT_SWP_DATACHANGED, -1, -1,    \
	(wxObjectEventFunction)(wxEventFunction)                    \
	(wxNotifyEventFunction) & fn, (wxObject*) NULL ),


BEGIN_EVENT_TABLE(TSwpSpectrum, wxPanel)
    EVT_PAINT(TSwpSpectrum::OnPaintEvent)
    EVT_SIZE(TSwpSpectrum::OnSizeEvent)
    EVT_ERASE_BACKGROUND(TSwpSpectrum::OnEraseBackgroundEvent)
    EVT_SWP_DATACHANGED(TSwpSpectrum::OnDataChanged)
    EVT_LEFT_DOWN(TSwpSpectrum::OnMouseDown)
    EVT_LEFT_UP(TSwpSpectrum::OnMouseUp)
    EVT_MOTION(TSwpSpectrum::OnMouseMove)
    EVT_MOUSE_CAPTURE_LOST(TSwpSpectrum::OnMouseCaptureLost)
END_EVENT_TABLE()

TSwpSpectrum::TSwpSpectrum(wxWindow *Parent,wxWindowID Id,const wxPoint &Position,const wxSize &Size):
wxPanel(Parent,Id,Position,Size,wxBORDER_SUNKEN|wxFULL_REPAINT_ON_RESIZE),m_Mutex(wxMUTEX_RECURSIVE)
{
    m_Data=NULL;
    m_DisplayData=NULL;
    m_Sweeping=false;
    m_DataPosition=0;
    m_DisplayDataPosition=0;
    m_ValidDisplayDataCount=0;
    m_StartFrequency=0;
    m_EndFrequency=0;
    m_RBW=0;
    m_MinLevel=-140;
    m_TuneWhenMouseMove=false;
    m_ChangedDataCount=0;
    m_InvalidDisplayData=false;

    Set(10000000,100000000,1000);
}

TSwpSpectrum::~TSwpSpectrum(void)
{
    if(m_Data)
    {
        free(m_Data);
    }
}

void TSwpSpectrum::Clear(void)
{
    if(m_Data!=NULL)
    {
        m_Mutex.Lock();

        if(m_Data)
        {
            free(m_Data);
            m_Data=NULL;
            m_DisplayData=NULL;
        }

        m_DataPosition=0;
        m_ValidDisplayDataCount=0;
        m_DisplayDataPosition=0;

        m_Mutex.Unlock();

        Refresh(false);
    }
}

bool TSwpSpectrum::Enable(bool enable)
{
    if(!enable)
    {
        m_TuneWhenMouseMove=false;
    }

    enable=wxPanel::Enable(enable);

    Refresh(false);

    return enable;
}

void TSwpSpectrum::Set(int64_t StartFrequency,int64_t EndFrequency,double RBW)
{
    if(m_StartFrequency!=StartFrequency || m_EndFrequency!=EndFrequency || m_RBW!=RBW)
    {
        m_StartFrequency=StartFrequency;
        m_EndFrequency=EndFrequency;
        m_RBW=RBW;
        m_Range=EndFrequency-StartFrequency+1;

        m_DataCount=(uint32_t)((long double)m_Range/m_RBW);

        UpdateParams();

        Clear();
    }
}

void TSwpSpectrum::Start(int64_t StartFrequency,int64_t EndFrequency,double RBW)
{
    Set(StartFrequency,EndFrequency,RBW);

    m_DataPosition=0;
    m_Sweeping=true;

    Refresh(false);
}

void TSwpSpectrum::Stop(void)
{
    m_Sweeping=false;
    Refresh(false);
}

void TSwpSpectrum::OnPaintEvent(wxPaintEvent &Event)
{
 wxBufferedPaintDC dc(this);
 int64_t Frequency;
 double fx,fy;
 double Step;
 int x,y;
 wxString s;
 wxSize Size;
 int Level;
 int First,Last,Count;
 wxRect Rect;

    dc.SetPen(*wxBLACK_PEN);
    dc.SetBrush(*wxBLACK_BRUSH);
    dc.DrawRectangle(GetClientRect());



    if(IsEnabled())
    {
        dc.SetPen(wxPen(wxColour(0xFF,0xFF,0xFF),1,wxSOLID));
    }
    else
    {
        dc.SetPen(wxPen(wxColour(0x80,0x80,0x80),1,wxSOLID));
    }

    if(IsEnabled())
    {
        dc.SetTextForeground(wxColour(0xFF,0xFF,0xFF));
    }
    else
    {
        dc.SetTextForeground(wxColour(0x80,0x80,0x80));
    }

    dc.SetTextBackground(wxColour(0,0,0));
    dc.SetFont(GetFont());

    Frequency=(int64_t)(m_StartFrequency/m_FrequencyStep)*m_FrequencyStep;
    fx=(Frequency-m_StartFrequency)*m_Frequency2Pixel+m_GraphRect.GetLeft();
    Step=m_FrequencyStep*m_Frequency2Pixel;

    for(;;)
    {
        x=(int)(fx+0.5);

        if(x>m_GraphRect.GetRight())
        {
            break;
        }

        if(x>=m_GraphRect.GetLeft())
        {
            dc.DrawLine(x,m_GraphRect.GetBottom(),x,m_GraphRect.GetBottom()+2);

            s=FormatFrequency(Frequency);

            Size=dc.GetTextExtent(s);

            x-=Size.GetWidth()>>1;

            if(x<m_GraphRect.GetLeft())
            {
                x=m_GraphRect.GetLeft();
            }
            else if(x+Size.GetWidth()>m_GraphRect.GetRight())
            {
                x=m_GraphRect.GetRight()-Size.GetWidth();
            }

            dc.DrawText(s,x,m_GraphRect.GetBottom()+2);
        }

        fx+=Step;
        Frequency+=m_FrequencyStep;
    }

    //level
    fy=m_GraphRect.GetTop();
    Step=m_LevelStep*m_Level2Pixel;
    Level=0;

    for(;;)
    {
        y=(int)(fy+0.5);

        if(y>m_GraphRect.GetBottom())
        {
            break;
        }

        dc.DrawLine(m_GraphRect.GetLeft(),y,m_GraphRect.GetLeft()-2,y);

        s.sprintf(wxT("%d"),Level);

        Size=dc.GetTextExtent(s);

        dc.DrawText(s,m_GraphRect.GetLeft()-2-Size.GetWidth(),y-Size.GetHeight()/2);

        fy+=Step;
        Level-=m_LevelStep;
    }

    if(IsEnabled())
    {
        dc.SetPen(wxPen(wxColour(0xE0,0xE0,0xE0),1,wxSOLID));
    }
    else
    {
        dc.SetPen(wxPen(wxColour(0x70,0x70,0x70),1,wxSOLID));
    }

    dc.DrawLine(m_GraphRect.GetTopLeft(),m_GraphRect.GetBottomLeft());
    dc.DrawLine(m_GraphRect.GetBottomLeft(),m_GraphRect.GetBottomRight());


    if(IsEnabled())
    {
        if(m_Sweeping)
        {
            dc.SetPen(wxPen(wxColour(0xff,0,0)));

            x=m_DisplayDataPosition*m_RBW*m_Frequency2Pixel+m_GraphRect.GetLeft();

            dc.DrawLine(x,m_GraphRect.GetTop(),x,m_GraphRect.GetBottom());
        }

        if(m_ValidDisplayDataCount)
        {
            wxRegionIterator RegionIterator(GetUpdateRegion());

            while(RegionIterator.HaveRects())
            {
                Rect=RegionIterator.GetRect();

                First=(int)((Rect.GetLeft()-m_GraphRect.GetLeft())*m_Pixel2Frequency/m_RBW);
                Last=(int)((Rect.GetRight()-m_GraphRect.GetLeft()+1)*m_Pixel2Frequency/m_RBW)-1;

                if(First<(int)m_DataCount && Last>=0 && First<(int)m_ValidDisplayDataCount)
                {
                    if(First<0)
                    {
                        First=0;
                    }

                    Count=Last-First+1;

                    if((First+Count)>(int)m_ValidDisplayDataCount)
                    {
                        Count=m_ValidDisplayDataCount-First;
                    }

                    dc.SetClippingRegion(m_GraphRect);
                    dc.SetPen(wxPen(wxColour(0x30,0xE0,0x0),1,wxSOLID));
                    dc.DrawLines(Count,m_DisplayData+First);
                }

                RegionIterator++;
            }
        }
    }
}

void TSwpSpectrum::OnSizeEvent(wxSizeEvent &Event)
{
    UpdateParams();

    if(IsShown())
    {
        if(m_ValidDisplayDataCount)
        {
            UpdateDisplayData(0,m_ValidDisplayDataCount);
        }
    }
    else
    {
        m_InvalidDisplayData=true;
    }

    Event.Skip();
}

void TSwpSpectrum::OnEraseBackgroundEvent(wxEraseEvent &Event)
{
}

//void OnData(wxNotifyEvent &Event);
void TSwpSpectrum::OnMouseDown(wxMouseEvent &Event)
{
    m_TuneWhenMouseMove=true;

    CaptureMouse();

    Tune(Event.GetPosition());

    Event.Skip();
}

void TSwpSpectrum::OnMouseUp(wxMouseEvent &Event)
{
    if(m_TuneWhenMouseMove)
    {
        m_TuneWhenMouseMove=false;
        ReleaseMouse();
    }

    Event.Skip();
}

void TSwpSpectrum::OnMouseMove(wxMouseEvent &Event)
{
    if(m_TuneWhenMouseMove)
    {
        Tune(Event.GetPosition());
    }

    Event.Skip();
}

void TSwpSpectrum::OnMouseCaptureLost(wxMouseCaptureLostEvent &Event)
{
    m_TuneWhenMouseMove=false;
}

void TSwpSpectrum::UpdateParams(void)
{
 wxClientDC dc(this);
 wxSize Size;
 double Step;
 int Dec;
 double Num;

    dc.SetFont(GetFont());
    Size=dc.GetTextExtent(wxT("-999"));

    m_GraphRect.SetTopLeft(wxPoint(Size.GetWidth()+2+2,Size.GetHeight()/2+2));
    m_GraphRect.SetBottomRight(wxPoint(GetClientSize().GetWidth()-5,GetClientSize().GetHeight()-Size.GetHeight()-2-2));

    if(m_GraphRect.GetWidth()<10)
    {
        m_GraphRect.SetWidth(10);
    }

    if(m_GraphRect.GetHeight()<10)
    {
        m_GraphRect.SetHeight(10);
    }

    m_Frequency2Pixel=(double)(m_GraphRect.GetWidth()-2)/m_Range;
    m_Pixel2Frequency=(double)m_Range/(m_GraphRect.GetWidth()-2);

    //calculate freq grid steps
    Step=m_Pixel2Frequency*95;

    Dec=(int)pow(10.0,(int)log10(Step));
    Num=(double)Step/(double)Dec;

    if(Num>5.0)
    {
        m_FrequencyStep=10;
    }
    else if(Num>2.0)
    {
        m_FrequencyStep=5;
    }
    else
    {
        m_FrequencyStep=2;
    }

    m_FrequencyStep*=Dec;

    m_Level2Pixel=(double)(m_GraphRect.GetHeight()-1)/-m_MinLevel;
    m_Pixel2Level=1.0/m_Level2Pixel;

    m_LevelStep=5;

    while(m_LevelStep*m_Level2Pixel<=(Size.GetHeight()))
    {
        if(m_LevelStep==5)
        {
            m_LevelStep=10;
        }
        else
        {
            m_LevelStep+=10;
        }
    }
}

void TSwpSpectrum::UpdateDisplayData(uint32_t First,uint32_t Count)
{
 uint32_t i;
 double k;
 uint32_t Last;

    k=m_RBW*m_Frequency2Pixel;

    Last=First+Count-1;

    for(i=First;i<=Last;i++)
    {
        m_DisplayData[i].x=m_GraphRect.GetLeft()+i*k;
        m_DisplayData[i].y=m_GraphRect.GetBottom()-(m_Data[i]-m_MinLevel)*m_Level2Pixel;
    }
}

void TSwpSpectrum::AddData(const float *Data,uint32_t Count)
{
 uint32_t Remain;

    m_Mutex.Lock();

    Remain=m_DataCount-m_DataPosition;

    if(Remain<Count)
    {
        Count=Remain;
    }

    if(m_Data==NULL)
    {
        m_Data=(float*)malloc(m_DataCount*(sizeof(float)+sizeof(wxPoint)));

        if(m_Data==NULL)
        {
            m_Mutex.Unlock();
            return;
        }

        m_DisplayData=(wxPoint*)(m_Data+m_DataCount);
    }

    memcpy(m_Data+m_DataPosition,Data,Count*sizeof(float));

    m_DataPosition=(m_DataPosition+Count)%m_DataCount;

    if(m_ChangedDataCount==0)
    {
        wxNotifyEvent Event(wxEVT_SWP_DATACHANGED);
        AddPendingEvent(Event);
    }

    m_ChangedDataCount+=Count;

    m_Mutex.Unlock();
}

void TSwpSpectrum::OnDataChanged(wxNotifyEvent &Event)
{
 uint32_t Remain;
 wxRect Rect;
 int x1,x2;

    m_Mutex.Lock();

    if(m_ChangedDataCount!=0)
    {
        Remain=m_DataCount-m_DisplayDataPosition;

        if(m_ChangedDataCount>m_DataCount)
        {
            m_ChangedDataCount=m_DataCount;
        }

        if(IsShown())
        {
            Rect=m_GraphRect;

            if(Remain<m_ChangedDataCount)
            {
                UpdateDisplayData(m_DisplayDataPosition,Remain);
                UpdateDisplayData(0,m_ChangedDataCount-Remain);

                x2=(int)((m_ChangedDataCount-Remain)*m_RBW*m_Frequency2Pixel)+1;

                Rect.SetWidth(x2+1);

                Refresh(false,&Rect);

                x1=(int)(m_DisplayDataPosition*m_RBW*m_Frequency2Pixel);
                x2=(int)((m_DisplayDataPosition+Remain)*m_RBW*m_Frequency2Pixel)+1;

                Rect.SetWidth(x2-x1+1);
                Rect.SetLeft(x1+m_GraphRect.GetLeft());

                Refresh(false,&Rect);
            }
            else
            {
                UpdateDisplayData(m_DisplayDataPosition,m_ChangedDataCount);

                x1=(int)(m_DisplayDataPosition*m_RBW*m_Frequency2Pixel);
                x2=(int)((m_DisplayDataPosition+m_ChangedDataCount)*m_RBW*m_Frequency2Pixel)+1;

                Rect.SetWidth(x2-x1+1);
                Rect.SetLeft(x1+m_GraphRect.GetLeft());

                Refresh(false,&Rect);
            }
        }
        else
        {
            m_InvalidDisplayData=true;
        }

        if(m_ValidDisplayDataCount<m_DataCount)
        {
            m_ValidDisplayDataCount+=m_ChangedDataCount;

            if(m_ValidDisplayDataCount>m_DataCount)
            {
                m_ValidDisplayDataCount=m_DataCount;
            }
        }

        m_DisplayDataPosition=(m_DisplayDataPosition+m_ChangedDataCount)%m_DataCount;

        m_ChangedDataCount=0;
    }

    m_Mutex.Unlock();
}

bool TSwpSpectrum::Show(bool show)
{
    show=wxPanel::Show(show);

    if(IsShown())
    {
        if(m_InvalidDisplayData)
        {
            m_Mutex.Lock();

            if(m_ValidDisplayDataCount)
            {
                UpdateDisplayData(0,m_ValidDisplayDataCount);
            }

            m_Mutex.Unlock();

            m_InvalidDisplayData=false;
        }
    }

    return show;
}

void TSwpSpectrum::Tune(const wxPoint &Pos)
{
 int64_t Frequency;

    if(!m_Sweeping && m_GraphRect.Contains(Pos))
    {
        Frequency=(Pos.x-m_GraphRect.GetLeft())*m_Pixel2Frequency+m_StartFrequency;

        TSpectrumTuneEvent Event(GetId(),Frequency);

        ProcessEvent(Event);
    }
}
