/* ***********************************************************************/
/*                                                                       */
/*                      WiNRADiO G39DDC EXCELSIOR                        */
/*         Header file for G39DDC device driver (PCIe and USB)           */
/*                     (C) Copyright 2015 RADIXON.                       */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G39DDC_H__
#define __G39DDC_H__

#ifdef HAVE_CONFIG_H
#include <linux/config.h>
#endif

#define __G39_USB__
#define __G39_PCI__
//#define __G39_DEBUG__

#include <linux/version.h>
#include <linux/types.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/uaccess.h>
#include <linux/time.h>
#include <linux/mm.h>
#include <linux/vmalloc.h>
#include <linux/kthread.h>
#include <linux/timer.h>

#ifdef CONFIG_COMPAT
#include <linux/compat.h>
#endif //CONFIG_COMPAT

#ifdef __G39_USB__
#include <linux/usb.h>
#endif //__G39_USB__

#ifdef __G39_PCI__
#include <linux/pci.h>
#include <linux/interrupt.h>
#endif //__G39_PCI__

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(a,b,c) ((a<<16)|(b<<8)|c)
#endif

#if LINUX_VERSION_CODE>=KERNEL_VERSION(4,11,0)
#include <linux/sched/signal.h>
#endif

#if LINUX_VERSION_CODE<KERNEL_VERSION(2,6,27)
#include <asm/semaphore.h>
#else
#include <linux/semaphore.h>
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
typedef unsigned long uintptr_t;
#endif

#include "g39ddc_ioctl.h"
#include "g39ddc_ioctl_compat.h"

#define G39_DEVICE_NAME             "g39ddc"
#define G39_MAX_DEVICE_COUNT        32
#define G39_DESCRIPTION             "WiNRADiO G39DDC device driver"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)

#define g39_mutex                   mutex
#define g39_mutex_init(__m__)       mutex_init(__m__)
#define g39_mutex_destroy(__m__)    mutex_destroy(__m__)
#define g39_mutex_lock(__m__)       mutex_lock(__m__)
#define g39_mutex_unlock(__m__)     mutex_unlock(__m__)

#else

#define g39_mutex                   semaphore
#define g39_mutex_init(__m__)       sema_init(__m__,1)
#define g39_mutex_destroy(__m__)
#define g39_mutex_lock(__m__)       down(__m__)
#define g39_mutex_unlock(__m__)     up(__m__)

#endif

#pragma pack(push,1)

struct g39_io_buffer
{
    struct list_head        list_entry;
    uintptr_t               user_buffer;
    uint32_t                size;
    int                     page_count;
    struct page             **pages;
    int                     changed;

#ifdef __G39_USB__
    void                    *mem;
#endif //__G39_USB__

#ifdef __G39_PCI__
    struct scatterlist      *sg_list;
    int                     sg_segment_count;
    struct pci_dev          *sg_pci_dev;
    int                     sg_direction;
#endif //__G39_PCI__
};

struct g39_complete_io_buffers
{
    struct list_head    list;
    struct semaphore    semaphore;
    spinlock_t          spinlock;
};

struct g39_pending_io_buffers
{
    struct list_head    list;
    spinlock_t          spinlock;
};

struct g39_io_stream
{
    uint8_t                         active;
    uint32_t                        buffer_size;
    uint32_t                        bytes_per_sec;
    struct rw_semaphore             rw_semaphore;
    struct g39_pending_io_buffers   pending_buffers;
    struct g39_complete_io_buffers  complete_buffers;
    uint16_t                        id;
};

#pragma pack(pop)

struct g39ddc;

typedef void (*g39_func_delete_t)(struct g39ddc *dev);
typedef int (*g39_func_open_t)(struct g39ddc *dev);
typedef void (*g39_func_close_t)(struct g39ddc *dev);
typedef int (*g39_func_set_power_t)(struct g39ddc *dev,uint32_t power);
typedef int (*g39_func_configure_fpga_t)(struct g39ddc *dev,void __user *user_buffer,uint32_t size);
typedef int (*g39_func_set_noise_blanker_t)(struct g39ddc *dev,uint16_t value);

typedef int (*g39_func_put_buffer_t)(struct g39ddc *dev,uint32_t stream_index,struct g39_io_buffer *io_buffer);

typedef int (*g39_func_if_start_t)(struct g39ddc *dev,uint16_t period);
typedef void (*g39_func_if_stop_t)(struct g39ddc *dev);
typedef int (*g39_func_get_adc_level_t)(struct g39ddc *dev,uint16_t *level);

typedef int (*g39_func_ddc1_start_t)(struct g39ddc *dev,uint32_t channel,uint32_t *coeffs);
typedef void (*g39_func_ddc1_stop_t)(struct g39ddc *dev,uint32_t channel);
typedef int (*g39_func_ddc1_update_params_t)(struct g39ddc *dev,uint32_t channel);

typedef int (*g39_func_ddc2_start_t)(struct g39ddc *dev,uint32_t channel,struct g39_ddc2_start_params *params);
typedef void (*g39_func_ddc2_stop_t)(struct g39ddc *dev,uint32_t channel);
typedef int (*g39_func_ddc2_set_frequency_t)(struct g39ddc *dev,uint32_t channel,uint32_t tuning_word);

typedef int (*g39_func_mmap_t)(struct g39ddc *dev,struct vm_area_struct *vma);
typedef int (*g39_func_set_spi_regs_t)(struct g39ddc *dev,struct g39_spi_register __user *user_buffer,uint16_t count,uint16_t id);
typedef int (*g39_func_set_spi2_regs_t)(struct g39ddc *dev,struct g39_spi2_register __user *user_buffer,uint16_t count,uint16_t id);
typedef int (*g39_func_usb_control1_t)(struct g39ddc *dev,const uint16_t *params,void __user *user_buffer,uint16_t buffer_size,void __user *transferred);
typedef int (*g39_func_usb_control2_t)(struct g39ddc *dev,void __user *user_buffer,uint16_t buffer_size,void __user *transferred);
typedef int (*g39_func_wd_t)(struct g39ddc *dev);
typedef int (*g39_func_swp_start_t)(struct g39ddc *dev);
typedef void (*g39_func_swp_stop_t)(struct g39ddc *dev);
typedef int (*g39_func_swp_snapshot_t)(struct g39ddc *dev);
typedef int (*g39_func_ddc1_playback_start_t)(struct g39ddc *dev,uint32_t channel,uint32_t divider);
typedef void (*g39_func_ddc1_playback_stop_t)(struct g39ddc *dev,uint32_t channel);
typedef int (*g39_func_dac_start_t)(struct g39ddc *dev,uint32_t dac_control);
typedef void (*g39_func_dac_stop_t)(struct g39ddc *dev);
typedef void (*g39_func_cancel_buffers_t)(struct g39ddc *dev,int stream_index);


struct g39ddc
{
    struct device                   dev;
    struct cdev                     cdev;
    uint32_t                        index;
    struct g39_state                state;
    struct g39_mutex                mutex;
    struct timespec                 last_power_off_time;
    uint8_t                         dac;

    struct device                   *p_dev;

    struct workqueue_struct         *workqueue;
    struct work_struct              wd_work;
    struct timer_list               wd_timer;
    uint32_t                        wd_counter;

    wait_queue_head_t               power_wait_queue;

    struct g39_io_stream            io_stream[G39_IO_STREAM_COUNT];
    struct g39_ddc1_params          ddc1_params[G39_CHANNEL_COUNT];

    uint8_t                         swp_active;
    uint16_t                        swp_settling_time;

    g39_func_delete_t               func_delete;
    g39_func_open_t                 func_open;
    g39_func_close_t                func_close;
    g39_func_set_power_t            func_set_power;
    g39_func_configure_fpga_t       func_configure_fpga;
    g39_func_set_spi_regs_t         func_set_spi_regs;
    g39_func_set_spi2_regs_t        func_set_spi2_regs;
    g39_func_set_noise_blanker_t    func_set_noise_blanker;

    g39_func_put_buffer_t           func_put_buffer;
    g39_func_if_start_t             func_if_start;
    g39_func_if_stop_t              func_if_stop;

    g39_func_get_adc_level_t        func_get_adc_level;

    g39_func_ddc1_start_t           func_ddc1_start;
    g39_func_ddc1_stop_t            func_ddc1_stop;
    g39_func_ddc1_update_params_t   func_ddc1_update_params;

    g39_func_ddc2_start_t           func_ddc2_start;
    g39_func_ddc2_stop_t            func_ddc2_stop;
    g39_func_ddc2_set_frequency_t   func_ddc2_set_frequency;

    g39_func_swp_start_t            func_swp_start;
    g39_func_swp_stop_t             func_swp_stop;
    g39_func_swp_snapshot_t         func_swp_snapshot;

    g39_func_ddc1_playback_start_t  func_ddc1_playback_start;
    g39_func_ddc1_playback_stop_t   func_ddc1_playback_stop;

    g39_func_dac_start_t            func_dac_start;
    g39_func_dac_stop_t             func_dac_stop;

    g39_func_cancel_buffers_t       func_cancel_buffers;

    g39_func_usb_control1_t         func_usb_control1_read;
    g39_func_usb_control1_t         func_usb_control1_write;
    g39_func_usb_control2_t         func_usb_control2_read;
    g39_func_usb_control2_t         func_usb_control2_write;
    g39_func_mmap_t                 func_mmap;
    g39_func_wd_t                   func_wd;

    uint8_t                         config[G39_CONFIG_SIZE];
};

#define G39_DEV_LOCK    g39_mutex_lock(&dev->mutex)
#define G39_DEV_UNLOCK  g39_mutex_unlock(&dev->mutex)

#define G39_DEV_LOCK2   g39_mutex_lock(&dev->g39_dev.mutex)
#define G39_DEV_UNLOCK2 g39_mutex_unlock(&dev->g39_dev.mutex)

#define G39_REG_LOCK    g39_mutex_lock(&g39_reg_mutex)
#define G39_REG_UNLOCK  g39_mutex_unlock(&g39_reg_mutex)

#define G39_IS_INPUT_STREAM(__index__) ((__index__)>=G39_IO_STREAM_IN_FIRST && (__index__)<=G39_IO_STREAM_IN_LAST)
#define G39_IS_OUTPUT_STREAM(__index__) ((__index__)>=G39_IO_STREAM_OUT_FIRST && (__index__)<=G39_IO_STREAM_OUT_LAST)

#if LINUX_VERSION_CODE>=KERNEL_VERSION(2,6,26)
#define G39_DEV_NAME(__dev__) dev_name(__dev__)
#else
#define G39_DEV_NAME(__dev__) (__dev__)->bus_id
#endif

#define G39_ERROR(__fmt__,...) printk(KERN_ERR G39_DEVICE_NAME ": " __fmt__,##__VA_ARGS__)
#define G39_INFO(__fmt__,...) printk(KERN_INFO G39_DEVICE_NAME ": " __fmt__,##__VA_ARGS__)

#define G39_DEV_ERROR(__dev__,__fmt__,...) printk(KERN_ERR G39_DEVICE_NAME " %s: " __fmt__,G39_DEV_NAME(__dev__),##__VA_ARGS__)
#define G39_DEV_INFO(__dev__,__fmt__,...) printk(KERN_INFO G39_DEVICE_NAME " %s: " __fmt__,G39_DEV_NAME(__dev__),##__VA_ARGS__)

#ifdef __G39_DEBUG__

#define DEBUG(__fmt__,...) printk(KERN_DEBUG G39_DEVICE_NAME ": " __fmt__,##__VA_ARGS__)
#define DEBUG_DEV(__dev__,__fmt__,...) printk(KERN_DEBUG G39_DEVICE_NAME " %s: " __fmt__,G39_DEV_NAME(__dev__),##__VA_ARGS__)

#else

#define DEBUG(__fmt__,...)
#define DEBUG_DEV(__dev__,__fmt__,...)


#endif //__G39_DEBUG__

#ifdef __G39_USB__

#define G39_USB_TIMEOUT                     5000

#define G39_USB_VID                         0x14E0
#define G39_USB_PID                         0x1109

#define G39_USB_EP_IN_FPGA                  0x02
#define G39_USB_EP_IN_CTRL                  0x01
#define G39_USB_EP_OUT_CTRL                 0x06
#define G39_USB_EP_OUT_FPGA                 0x08


#define USB_REG_CTRL1                   0x0000
#define USB_REG_FLUSH                   0x0001
#define USB_REG_COMMAND_ID              0x0002
#define USB_REG_SPI_LO                  0x0004
#define USB_REG_SPI_HI                  0x0005
#define USB_REG_IF_BUFF_CTRL            0x0008
#define USB_REG_NOISE_BLANKER_THRESHOLD 0x0009
#define USB_REG_COEFF_WR_CTRL           0x000C
#define USB_REG_DDC1_FIR_COEFF_0        0x000D
#define USB_REG_DDC1_FIR_COEFF_1        0x000E
#define USB_REG_DDC2_FIR_1_COEFF_0      0x000F
#define USB_REG_DDC2_FIR_1_COEFF_1      0x0010
#define USB_REG_DDC2_FIR_2_COEFF_0      0x0011
#define USB_REG_DDC2_FIR_2_COEFF_1      0x0012
#define USB_REG_DDC1_CTRL_0             0x0018
#define USB_REG_DDC1_TUNING_WORD_0      0x0019
#define USB_REG_DDC1_CIC_CTRL_0         0x001B
#define USB_REG_DDC1_FIR_CTRL_0         0x001C
#define USB_REG_DDC1_CTRL_1             0x0020
#define USB_REG_DDC1_TUNING_WORD_1      0x0021
#define USB_REG_DDC1_CIC_CTRL_1         0x0023
#define USB_REG_DDC1_FIR_CTRL_1         0x0024

#define USB_REG_DDC2_SHIFT_CONTROL_0    0x0028
#define USB_REG_DDC2_SHIFT_TUNING_0     0x0029
#define USB_REG_DDC2_FIR1_CONTROL1_0    0x002A
#define USB_REG_DDC2_FIR1_CONTROL2_0    0x002B
#define USB_REG_DDC2_FIR2_CONTROL1_0    0x002C
#define USB_REG_DDC2_FIR2_CONTROL2_0    0x002D
#define USB_REG_DDC2_RETIMER_0          0x002E
#define USB_REG_DDC2_INPUT_SWITCH_0     0x002F

#define USB_REG_DDC2_SHIFT_CONTROL_1    0x0032
#define USB_REG_DDC2_SHIFT_TUNING_1     0x0033
#define USB_REG_DDC2_FIR1_CONTROL1_1    0x0034
#define USB_REG_DDC2_FIR1_CONTROL2_1    0x0035
#define USB_REG_DDC2_FIR2_CONTROL1_1    0x0036
#define USB_REG_DDC2_FIR2_CONTROL2_1    0x0037
#define USB_REG_DDC2_RETIMER_1          0x0038
#define USB_REG_DDC2_INPUT_SWITCH_1     0x0039

#define USB_REG_DAC_CONTROL             0x003C

#define G39_USB_INPUT_BUFFER_MIN_SIZE       (8*1024)
#define G39_USB_INPUT_BUFFER_MAX_SIZE       (128*1024)
#define G39_USB_INPUT_BUFFER_COUNT          4
#define G39_USB_INPUT_SWP_BUFFER_SIZE       (16384)
#define G39_USB_OUTPUT_BUFFER_COUNT         4
#define G39_USB_OUTPUT_BUFFER_SIZE          (128*1024)

#define G39_USB_INPUT_PACKET_PAYLOAD_SIZE   (96*2)

#pragma pack(push,1)

struct g39_usb_input_packet
{
    uint8_t     sync[2];
    uint8_t     source;
    uint8_t     flags;
    uint16_t    id;
    uint8_t     data[G39_USB_INPUT_PACKET_PAYLOAD_SIZE];
};

struct g39_usb_output_packet
{
    uint16_t    sync;
    uint16_t    length;
    uint16_t    command;
    uint16_t    address;
    union
    {
        uint8_t     data8[0];
        uint16_t    data16[0];
        uint32_t    data32[0];
    };
};

#pragma pack(pop)

struct g39ddc_usb;

struct g39_usb_buffer
{
    struct urb              *urb;
    uint8_t                 *mem;
    struct list_head        list_entry;
    uint8_t                 pending;
    struct g39ddc_usb       *dev;
};

struct g39_usb_buffers
{
    struct list_head    list;
    spinlock_t          spinlock;
};

struct g39ddc_usb
{
    struct g39ddc                   g39_dev;
    struct usb_interface            *usb_interface;
    struct usb_device               *usb_dev;

    struct
    {
        struct task_struct          *thread;
        uint32_t                    stream_counter;
        struct g39_usb_buffer       buffer[G39_USB_INPUT_BUFFER_COUNT];
        struct g39_usb_buffers      complete_usb_buffers;
        uint32_t                    buffer_size;
    }                               input;

    struct
    {
        struct task_struct          *thread;
        uint32_t                    stream_counter;
        struct g39_usb_buffer       buffer[G39_USB_OUTPUT_BUFFER_COUNT];
        struct g39_usb_buffers      avail_usb_buffers;
        uint32_t                    bytes_per_packet[G39_IO_STREAM_OUT_COUNT];
        uint32_t                    ddc1_divider[G39_CHANNEL_COUNT];
        uint8_t                     dac_control;
    }                               output;

    struct g39_io_buffer            *io_buffer[G39_IO_STREAM_COUNT];

};

#endif //__G39_USB__



#ifdef __G39_PCI__

#define G39_PCI_VENDOR                      0x1A39
#define G39_PCI_DEVICE                      0x0004
#define G39_PCI_SUBSYSTEM_VENDOR            0x186F
#define G39_PCI_SUBSYSTEM_DEVICE            0x1209

#define LCLK_DIVIN                          0
#define LCLK_DIVFB                          31
#define LCLK_DIVOUT                         7
#define L_PLL_RANGE                         0x3

#define G39_I2C_DIV_A                       1
#define G39_I2C_DIV_B                       (((125000000 / (22 * 390000 * (G39_I2C_DIV_A + 1))) - 1)&0x3F)
#define G39_I2C_MAX_RETRY                   3000000
#define G39_I2C_FIFO_SIZE                   8

#define PCI_SYS_CFG_SYSTEM                  0x800
#define LB_CTL                              0x804
#define CLK_CSR                             0x808
#define INT_STAT                            0x814
#define INT_CFG0                            0x820
#define INT_CFG1                            0x824
#define INT_CFG2                            0x828
#define INT_CFG3                            0x82C

#define TWI_CTRL                            0x900
#define TWI_STATUS                          0x904
#define TWI_ADDRESS                         0x908
#define TWI_DATA                            0x90C
#define TWI_IRT_STATUS                      0x910
#define TWI_TR_SIZE                         0x914
#define TWI_TO                              0x91C
#define TWI_IR_EN                   0x924
#define TWI_IR_DIS                  0x928


#define GPIO_DIRECTION_MODE                 0xA04
#define GPIO_OUTPUT_ENABLE                  0xA08
#define GPIO_OUTPUT_VALUE                   0xA0C
#define GPIO_INPUT_VALUE                    0xA10

#define GPIO_INT_MASK_CLR                   0xA18
#define GPIO_INT_MASK_SET                   0xA1C
#define GPIO_INT_STATUS                     0xA20
#define GPIO_INT_TYPE                       0xA24
#define GPIO_INT_VALUE                      0xA28


#define FCL_CTRL                            0xB00
#define FCL_STATUS                          0xB04
#define FCL_IODATA_OUT                      0xB0C
#define FCL_EN                              0xB10
#define FCL_CLK_DIV                         0xB1C
#define FCL_IRQ                             0xB20

#define IRQ_I2C                             (1<<0)
#define IRQ_FCL                             (1<<1)
#define IRQ_GPIO                            (1<<15)

#define GPIO_BIT_TMP                        0xFE
#define GPIO_BIT_FPGA_RESET                 (1<<8)
#define GPIO_BIT_CTL                        (1<<9)
#define GPIO_BIT_TMP_ERROR                  (1<<10)
#define GPIO_BIT_DMA_IRQ                    (1<<11)
#define GPIO_BIT_FAN_ERROR                  (1<<12)
#define GPIO_BIT_FE_POWER                   (1<<14)
#define GPIO_BIT_POWER_STATE                (1<<15)

#define REG_SPI2_LO                 0x0018
#define REG_SPI2_HI                 0x001C
#define REG_SPI                     0x0020
#define REG_IF_BUFF_CTRL            0x0024
#define REG_ADC_LEVEL               0x0028
#define REG_DATA_LIMIT              0x002C
#define REG_DAC_CONTROL             0x0030
#define REG_DMA_FIFO_RESET          0x003C
#define REG_DMA_CONFIG              0x0040
#define REG_DMA_ENABLE_OUT          0x0044
#define REG_DMA_ENABLE_IN           0x0048
#define REG_DMA_IRQ_STATUS_OUT      0x004C
#define REG_DMA_IRQ_STATUS_IN       0x0050

#define REG_DMA_PTR_OUT_BASE        0x0054
#define REG_DMA_PTR_IN_BASE         (REG_DMA_PTR_OUT_BASE+3*4*4)

#define REG_FIR_COEFF               0x00F4
#define REG_DDC1_CTRL1              0x00F8
#define REG_DDC1_TUNING_WORD        0x00FC
#define REG_DDC1_START_PHASE        0x0100
#define REG_DDC1_CIC_CTRL1          0x0104
#define REG_DDC1_FIR_CTRL1          0x0108

#define REG_DDC1_COUNT              5

#define REG_DDC2_SHIFT_CONTROL      0x0120
#define REG_DDC2_SHIFT_TUNING       0x0124
#define REG_DDC2_FIR1_CONTROL1      0x0128
#define REG_DDC2_FIR1_CONTROL2      0x012C
#define REG_DDC2_FIR2_CONTROL1      0x0130
#define REG_DDC2_FIR2_CONTROL2      0x0134
#define REG_DDC2_RETIMER            0x0138
#define REG_DDC2_INPUT_SWITCH       0x013C
//0x013C - 0x0168 reserved

#define REG_DDC2_COUNT              19


#define G39_DMA_IN_IF               0
#define G39_DMA_IN_CH1_DDC1         1
#define G39_DMA_IN_CH2_DDC1         2
#define G39_DMA_IN_CH1_DDC2         3
#define G39_DMA_IN_CH2_DDC2         4

#define G39_DMA_OUT_CH1_DDC1        0
#define G39_DMA_OUT_CH2_DDC1        1
#define G39_DMA_OUT_DAC             2

#define G39_DMA_CHANNEL_OUT_COUNT   G39_IO_STREAM_OUT_COUNT
#define G39_DMA_CHANNEL_IN_COUNT    G39_IO_STREAM_IN_COUNT

#define G39_DMA_PAGE_SIZE           4096

#ifndef DMA_BIT_MASK
#define DMA_BIT_MASK(n)             (((n) == 64)?~0ULL:((1ULL<<(n))-1))
#endif //DMA_BIT_MASK



#pragma pack(push,1)

struct g39_sg_segment_32
{
    uint32_t    address;
    uint32_t    length;
};

struct g39_sg_segment_64
{
    uint64_t    address;
    uint32_t    length;
};

#pragma pack(pop)

struct g39_dma_subchannel
{
    struct g39_io_buffer    *io_buffer;
    dma_addr_t              aligned_sg_mem;
    void                    *aligned_sg_mem_cpu;
    dma_addr_t              sg_mem;
    void                    *sg_mem_cpu;
};

struct g39_dma_channel
{
    uint8_t                     out;
    uint32_t                    index;
    struct g39_dma_subchannel   subchannel[2];
    uint32_t                    subchannel_index;
    struct g39_mutex            mutex;
};

struct g39ddc_pci
{
    struct g39ddc           g39_dev;
    struct pci_dev          *pci_dev;
    uint8_t                 dma_64;
    uint8_t                 *bar0;
    uint8_t                 *bar4;
    uint32_t                dma_status_in;
    uint32_t                dma_subchannel_mask_in;
    uint32_t                dma_status_out;
    uint32_t                dma_subchannel_mask_out;
    uint32_t                dma_enabled_in;
    uint32_t                dma_enabled_out;
    spinlock_t              irq_spinlock;
    struct work_struct      irq_work;
    uint32_t                dma_sg_list_memory_size;
    uint32_t                dma_max_buffer_size;
    uint8_t                 msi;
    uint8_t                 disabled;

    struct g39_dma_channel  dma_channel[G39_IO_STREAM_COUNT];
};

#define bar0_write32(__offset__,__value__) iowrite32(__value__,dev->bar0+(__offset__))
#define bar0_read32(__offset__) ioread32(dev->bar0+(__offset__))
#define bar4_write32(__offset__,__value__) iowrite32(__value__,dev->bar4+(__offset__))
#define bar4_read32(__offset__) ioread32(dev->bar4+(__offset__))

#define gpio_set(__bits__) bar4_write32(GPIO_OUTPUT_VALUE,bar4_read32(GPIO_OUTPUT_VALUE) | (__bits__));bar4_read32(GPIO_OUTPUT_VALUE)
#define gpio_clr(__bits__) bar4_write32(GPIO_OUTPUT_VALUE,bar4_read32(GPIO_OUTPUT_VALUE) & (~(__bits__)));bar4_read32(GPIO_OUTPUT_VALUE)

#endif //__G39_PCI__

#endif //__G39DDC_H__
