/* ***********************************************************************/
/*                                                                       */
/*                      WiNRADiO G39DDC EXCELSIOR                        */
/*         Ioctl codes and definitions for G39DDC device driver          */
/*                     (C) Copyright 2015 RADIXON.                       */
/*                                                                       */
/* ***********************************************************************/

#ifndef __G39_IOCTL_H__
#define __G39_IOCTL_H__

#define G39_INTERFACE_TYPE_PCIE         0
#define G39_INTERFACE_TYPE_USB          1

#define G39_CHANNEL_COUNT               2

#define G39_IO_STREAM_IN_IF             0
#define G39_IO_STREAM_IN_DDC1_0         1
#define G39_IO_STREAM_IN_DDC1_1         2
#define G39_IO_STREAM_IN_DDC2_0         3
#define G39_IO_STREAM_IN_DDC2_1         4
#define G39_IO_STREAM_OUT_DDC1_0        5
#define G39_IO_STREAM_OUT_DDC1_1        6
#define G39_IO_STREAM_OUT_DAC           7
#define G39_IO_STREAM_COUNT             8

#define G39_IO_STREAM_IN_FIRST          G39_IO_STREAM_IN_IF
#define G39_IO_STREAM_IN_LAST           G39_IO_STREAM_IN_DDC2_1
#define G39_IO_STREAM_IN_COUNT          (G39_IO_STREAM_IN_LAST-G39_IO_STREAM_IN_FIRST+1)

#define G39_IO_STREAM_OUT_FIRST         G39_IO_STREAM_OUT_DDC1_0
#define G39_IO_STREAM_OUT_LAST          G39_IO_STREAM_OUT_DAC
#define G39_IO_STREAM_OUT_COUNT         (G39_IO_STREAM_OUT_LAST-G39_IO_STREAM_OUT_FIRST+1)

#define G39_CONFIG_SIZE                 8196
#define G39_MAX_BUFFER_SIZE             6258688

#pragma pack(push,1)

struct g39_usb_control1
{
    uint16_t    params[3];
    void        *buffer;
    uint16_t    size;
    uint16_t    *transferred;
};

struct g39_usb_control2
{
    void        *buffer;
    uint16_t    size;
    uint16_t    *transferred;
};

struct g39_fpga_data
{
    void        *buffer;
    uint32_t    size;
};

struct g39_spi_register
{
    uint16_t    address;
    uint32_t    value;
};

struct g39_spi_registers
{
    uint16_t                    count;
    uint16_t                    id;
    struct g39_spi_register     *regs;
};

struct g39_spi2_register
{
    uint32_t    u32[2];
};

struct g39_spi2_registers
{
    uint16_t                    count;
    uint16_t                    id;
    struct g39_spi2_register    *regs;
};

struct g39_ddc1_params
{
    uint16_t    control_word;
    uint16_t    cic1;
    uint16_t    cic2;
    uint16_t    fir_control;
    uint16_t    fir_len;
    uint32_t    tuning_word;
};

struct g39_ddc1_start_params
{
    uint32_t                buffer_size;
    uint32_t                bytes_per_sec;
    struct g39_ddc1_params  params;
    uint32_t                *coeffs;
};

struct g39_ddc2_fir
{
    union
    {
        uint32_t        u32;

        struct
        {
            uint32_t    enabled:1;
            uint32_t    bypass:1;
            uint32_t    mac_length:9;
            uint32_t    ratio_up:5;
            uint32_t    ratio_down:5;
            uint32_t    scale:5;
            uint32_t    reserved:6;
        };
    } ctrl1;

    union
    {
        uint32_t        u32;

        struct
        {
            uint32_t    len:10;
            uint32_t    reserved:22;
        };
    } ctrl2;

    uint32_t coeffs[512];
};

struct g39_ddc2_start_params
{
    uint32_t            buffer_size;
    uint32_t            bytes_per_sec;

    struct g39_ddc2_fir fir1;
    struct g39_ddc2_fir fir2;

    union
    {
        uint32_t        u32;

        struct
        {
            uint32_t    enabled:1;
            uint32_t    bypass:1;
            uint32_t    divider:12;
            uint32_t    reserved:18;
        };
    } retimer;
};

struct g39_ddc1_playback_params
{
    uint32_t    buffer_size;
    uint32_t    bytes_per_sec;
    uint32_t    divider;
};

struct g39_dac_params
{
    uint32_t    buffer_size;
    uint32_t    bytes_per_sec;
    uint32_t    dac_control;
};

struct g39_state
{
    uint8_t connected:1;
    uint8_t open:1;
    uint8_t power:1;
    uint8_t suspended:1;
    uint8_t temperature:1;
    uint8_t fpga:1;
};

#pragma pack(pop)

#define G39_IOCTL_GET_CONFIG                    _IOR(0x8C,0x00,uint8_t[G39_CONFIG_SIZE])
#define G39_IOCTL_GET_STATE                     _IOR(0x8C,0x01,struct g39_state)
#define G39_IOCTL_SET_POWER                     _IOW(0x8C,0x02,uint8_t)
#define G39_IOCTL_CONFIGURE_FPGA                _IOW(0x8C,0x03,struct g39_fpga_data)
#define G39_IOCTL_SET_SPI_REGS                  _IOW(0x8C,0x04,struct g39_spi_registers)
#define G39_IOCTL_SET_SPI2_REGS                 _IOW(0x8C,0x05,struct g39_spi2_registers)
#define G39_IOCTL_SET_NOISE_BLANKER             _IOW(0x8C,0x06,uint16_t)
#define G39_IOCTL_IF_START                      _IOW(0x8C,0x07,uint16_t)
#define G39_IOCTL_IF_STOP                       _IO(0x8C,0x08)
#define G39_IOCTL_IF_PUT_BUFFER                 _IOW(0x8C,0x09,void*)
#define G39_IOCTL_IF_GET_BUFFER                 _IOR(0x8C,0x0A,void*)
#define G39_IOCTL_GET_ADC_LEVEL                 _IOR(0x8C,0x0B,uint16_t)
#define G39_IOCTL_DDC1_0_START                  _IOW(0x8C,0x0C,struct g39_ddc1_start_params)
#define G39_IOCTL_DDC1_0_STOP                   _IO(0x8C,0x0D)
#define G39_IOCTL_DDC1_0_SET_FREQUENCY          _IOW(0x8C,0x0E,uint32_t)
#define G39_IOCTL_DDC1_0_PUT_BUFFER             _IOW(0x8C,0x0F,void*)
#define G39_IOCTL_DDC1_0_GET_BUFFER             _IOR(0x8C,0x10,void*)
#define G39_IOCTL_DDC1_1_START                  _IOW(0x8C,0x11,struct g39_ddc1_start_params)
#define G39_IOCTL_DDC1_1_STOP                   _IO(0x8C,0x12)
#define G39_IOCTL_DDC1_1_SET_FREQUENCY          _IOW(0x8C,0x13,uint32_t)
#define G39_IOCTL_DDC1_1_PUT_BUFFER             _IOW(0x8C,0x14,void*)
#define G39_IOCTL_DDC1_1_GET_BUFFER             _IOR(0x8C,0x15,void*)

#define G39_IOCTL_DDC2_0_START                  _IOW(0x8C,0x16,struct g39_ddc2_start_params)
#define G39_IOCTL_DDC2_0_STOP                   _IO(0x8C,0x17)
#define G39_IOCTL_DDC2_0_SET_FREQUENCY          _IOW(0x8C,0x18,uint32_t)
#define G39_IOCTL_DDC2_0_PUT_BUFFER             _IOW(0x8C,0x19,void*)
#define G39_IOCTL_DDC2_0_GET_BUFFER             _IOR(0x8C,0x1A,void*)
#define G39_IOCTL_DDC2_1_START                  _IOW(0x8C,0x1B,struct g39_ddc2_start_params)
#define G39_IOCTL_DDC2_1_STOP                   _IO(0x8C,0x1C)
#define G39_IOCTL_DDC2_1_SET_FREQUENCY          _IOW(0x8C,0x1D,uint32_t)
#define G39_IOCTL_DDC2_1_PUT_BUFFER             _IOW(0x8C,0x1E,void*)
#define G39_IOCTL_DDC2_1_GET_BUFFER             _IOR(0x8C,0x1F,void*)

#define G39_IOCTL_SWP_START                     _IOW(0x8C,0x28,uint16_t)
#define G39_IOCTL_SWP_STOP                      _IO(0x8C,0x29)
#define G39_IOCTL_SWP_PUT_BUFFER                _IOW(0x8C,0x2A,void*)
#define G39_IOCTL_SWP_GET_BUFFER                _IOR(0x8C,0x2B,void*)
#define G39_IOCTL_SWP_SNAPSHOT                  _IO(0x8C,0x2C)

#define G39_IOCTL_DDC1_0_PLAYBACK_START         _IOW(0x8C,0x30,struct g39_ddc1_playback_params)
#define G39_IOCTL_DDC1_0_PLAYBACK_STOP          _IO(0x8C,0x31)
#define G39_IOCTL_DDC1_0_PLAYBACK_PUT_BUFFER    _IOW(0x8C,0x32,void*)
#define G39_IOCTL_DDC1_0_PLAYBACK_GET_BUFFER    _IOR(0x8C,0x33,void*)

#define G39_IOCTL_DDC1_1_PLAYBACK_START         _IOW(0x8C,0x34,struct g39_ddc1_playback_params)
#define G39_IOCTL_DDC1_1_PLAYBACK_STOP          _IO(0x8C,0x35)
#define G39_IOCTL_DDC1_1_PLAYBACK_PUT_BUFFER    _IOW(0x8C,0x36,void*)
#define G39_IOCTL_DDC1_1_PLAYBACK_GET_BUFFER    _IOR(0x8C,0x37,void*)

#define G39_IOCTL_DAC_START                     _IOW(0x8C,0x38,struct g39_dac_params)
#define G39_IOCTL_DAC_STOP                      _IO(0x8C,0x39)
#define G39_IOCTL_DAC_PUT_BUFFER                _IOW(0x8C,0x40,void*)
#define G39_IOCTL_DAC_GET_BUFFER                _IOR(0x8C,0x41,void*)

#define G39_IOCTL_IF_CANCEL_BUFFERS             _IO(0x8C,0x50)
#define G39_IOCTL_DDC1_0_CANCEL_BUFFERS         _IO(0x8C,0x51)
#define G39_IOCTL_DDC1_1_CANCEL_BUFFERS         _IO(0x8C,0x52)
#define G39_IOCTL_DDC2_0_CANCEL_BUFFERS         _IO(0x8C,0x53)
#define G39_IOCTL_DDC2_1_CANCEL_BUFFERS         _IO(0x8C,0x54)

#define G39_IOCTL_USB_CONTROL1_READ             _IOR(0x8C,0x60,struct g39_usb_control1)
#define G39_IOCTL_USB_CONTROL1_WRITE            _IOW(0x8C,0x61,struct g39_usb_control1)
#define G39_IOCTL_USB_CONTROL2_READ             _IOR(0x8C,0x62,struct g39_usb_control2)
#define G39_IOCTL_USB_CONTROL2_WRITE            _IOW(0x8C,0x63,struct g39_usb_control2)

#define G39_IOCTL_WAIT_FOR_POWER_OFF            _IO(0x8C,0x70)

#endif // __G39_IOCTL_H__
