#include "g39ddcapi.h"
#include <stdio.h>
#include <math.h>
#include <dlfcn.h>
#include <inttypes.h>
#include <pthread.h>
#include <pulse/simple.h>

G39DDC_OPEN_DEVICE OpenDevice;
G39DDC_CLOSE_DEVICE CloseDevice;
G39DDC_SET_POWER SetPower;
G39DDC_SET_DDC1 SetDDC1;
G39DDC_GET_DDC1 GetDDC1;
G39DDC_SET_DDC2 SetDDC2;
G39DDC_GET_DDC2 GetDDC2;
G39DDC_GET_DDC_INFO GetDDCInfo;
G39DDC_SET_CALLBACKS SetCallbacks;
G39DDC_SET_FREQUENCY SetFrequency;
G39DDC_GET_FREQUENCY GetFrequency;
G39DDC_START_DDC1 StartDDC1;
G39DDC_STOP_DDC1 StopDDC1;
G39DDC_START_DDC2 StartDDC2;
G39DDC_STOP_DDC2 StopDDC2;
G39DDC_START_AUDIO StartAudio;
G39DDC_STOP_AUDIO StopAudio;
G39DDC_GET_AUDIO GetAudio;
G39DDC_GET_AUDIO_SAMPLE_RATE GetAudioSampleRate;
G39DDC_SET_AGC SetAGC;
G39DDC_SET_AGC_PARAMS SetAGCParams;
G39DDC_SET_DEMODULATOR_MODE SetDemodulatorMode;
G39DDC_SET_DEMODULATOR_FILTER_BANDWIDTH SetDemodulatorFilterBandwidth;
G39DDC_SET_DDC1_FREQUENCY SetDDC1Frequency;
G39DDC_SET_DDC2_FREQUENCY SetDDC2Frequency;
G39DDC_SET_DEMODULATOR_FREQUENCY SetDemodulatorFrequency;
G39DDC_GET_DDC1_FREQUENCY GetDDC1Frequency;
G39DDC_GET_DDC2_FREQUENCY GetDDC2Frequency;
G39DDC_GET_DEMODULATOR_FREQUENCY GetDemodulatorFrequency;
G39DDC_GET_FRONT_END_FREQUENCY GetFrontEndFrequency;


int32_t hDevice;
void *API=NULL;
pa_simple *PaSimple=NULL;

int Initialize(void)
{
 uint32_t AudioSampleRate;
 uint32_t AudioSampleRateIndex;
 pa_sample_spec SampleFormat;
 int Error;
 
    API=dlopen("libg39ddcapi.so",RTLD_LAZY);

    if(API==NULL)
    {
        printf("Failed to load libg39ddcapi.so. %s\n",dlerror());
        return 0;
    }

    //retrieve addresses of used API functions
    OpenDevice=(G39DDC_OPEN_DEVICE)dlsym(API,"OpenDevice");
    CloseDevice=(G39DDC_CLOSE_DEVICE)dlsym(API,"CloseDevice");
    SetPower=(G39DDC_SET_POWER)dlsym(API,"SetPower");
    SetDDC1=(G39DDC_SET_DDC1)dlsym(API,"SetDDC1");
    GetDDC1=(G39DDC_GET_DDC1)dlsym(API,"GetDDC1");
    SetDDC2=(G39DDC_SET_DDC2)dlsym(API,"SetDDC2");
    GetDDC2=(G39DDC_GET_DDC2)dlsym(API,"GetDDC2");
    GetDDCInfo=(G39DDC_GET_DDC_INFO)dlsym(API,"GetDDCInfo");
    SetCallbacks=(G39DDC_SET_CALLBACKS)dlsym(API,"SetCallbacks");
    SetFrequency=(G39DDC_SET_FREQUENCY)dlsym(API,"SetFrequency");
    GetFrequency=(G39DDC_GET_FREQUENCY)dlsym(API,"GetFrequency");
    StartDDC1=(G39DDC_START_DDC1)dlsym(API,"StartDDC1");
    StopDDC1=(G39DDC_STOP_DDC1)dlsym(API,"StopDDC1");
    StartDDC2=(G39DDC_START_DDC2)dlsym(API,"StartDDC2");
    StopDDC2=(G39DDC_STOP_DDC2)dlsym(API,"StopDDC2");
    StartAudio=(G39DDC_START_AUDIO)dlsym(API,"StartAudio");
    StopAudio=(G39DDC_STOP_AUDIO)dlsym(API,"StopAudio");
    GetAudio=(G39DDC_GET_AUDIO)dlsym(API,"GetAudio");
    GetAudioSampleRate=(G39DDC_GET_AUDIO_SAMPLE_RATE)dlsym(API,"GetAudioSampleRate");
    SetAGC=(G39DDC_SET_AGC)dlsym(API,"SetAGC");
    SetAGCParams=(G39DDC_SET_AGC_PARAMS)dlsym(API,"SetAGCParams");
    SetDemodulatorMode=(G39DDC_SET_DEMODULATOR_MODE)dlsym(API,"SetDemodulatorMode");
    SetDemodulatorFilterBandwidth=(G39DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)dlsym(API,"SetDemodulatorFilterBandwidth");
    SetDDC1Frequency=(G39DDC_SET_DDC1_FREQUENCY)dlsym(API,"SetDDC1Frequency");
    SetDDC2Frequency=(G39DDC_SET_DDC2_FREQUENCY)dlsym(API,"SetDDC2Frequency");
    SetDemodulatorFrequency=(G39DDC_SET_DEMODULATOR_FREQUENCY)dlsym(API,"SetDemodulatorFrequency");
    GetDDC1Frequency=(G39DDC_GET_DDC1_FREQUENCY)dlsym(API,"GetDDC1Frequency");
    GetDDC2Frequency=(G39DDC_GET_DDC2_FREQUENCY)dlsym(API,"GetDDC2Frequency");
    GetDemodulatorFrequency=(G39DDC_GET_DEMODULATOR_FREQUENCY)dlsym(API,"GetDemodulatorFrequency");
    GetFrontEndFrequency=(G39DDC_GET_FRONT_END_FREQUENCY)dlsym(API,"GetFrontEndFrequency");    

    //open the first available G39DDC device
    hDevice=OpenDevice(G39DDC_OPEN_FIRST);

    if(hDevice<0)
    {
        printf("No available G39DDC device found.\n");
        return 0;
    }

    //retrieve current audio sample rate index of channel 0
    GetAudio(hDevice,0,&AudioSampleRateIndex);
    GetAudioSampleRate(hDevice,AudioSampleRateIndex,&AudioSampleRate);

    //initialize pulse audio
    SampleFormat.channels=2;
    SampleFormat.rate=AudioSampleRate;
    SampleFormat.format=PA_SAMPLE_FLOAT32LE;
    
    PaSimple=pa_simple_new(NULL,"G39DDC SDK EXAMPLE",PA_STREAM_PLAYBACK,NULL,"Audio0",&SampleFormat,NULL,NULL,&Error);
    
    if(PaSimple==NULL)
    {
        printf("Failed to initialize pulse audio.\n");
    }

    return 1;
}

void MyDDC2PreprocessedStreamCallback(uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS,uintptr_t UserData)
{
 double Slevel_dBm;

    if(Channel==0) 
    {
      //convert slevel RMS in Volts to dBm
      Slevel_dBm=10.0*log10(SlevelRMS*SlevelRMS*(1000.0/50.0));      
      printf("\rSlevel: RMS[V]=%9.7f V, RMS[dBm]=%6.1f dBm, Peak[V]=%9.7f V",SlevelRMS,Slevel_dBm,SlevelPeak);
      fflush(stdout);
    }
}

void MyAudioStreamCallback(uint32_t Channel,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples,uintptr_t UserData)
{
 int Error;
  
    if(Type==2 && PaSimple!=NULL)
    {
        //we want to play only audio buffers of type 2 which means final audio samples (audio gain, audio filter and volume are applied)
        pa_simple_write(PaSimple,Buffer,NumberOfSamples*2*sizeof(float),&Error);
    }
}

int main(void)
{
 G39DDC_DDC_INFO DDCInfo;
 uint32_t DDCTypeIndex;
 G39DDC_CALLBACKS Callbacks;
 uint64_t Frequency;
 int32_t DDC1Frequency;
 int32_t DDC2Frequency;
 int32_t DemodulatorFrequency;
 uint64_t FrontEndFrequency;
 uint32_t SamplesPerBuffer;
 uint32_t AudioSampleRateIndex;
 uint32_t AudioSampleRate;

    if(!Initialize())
    {
        return -1;
    }

    //set power
    SetPower(hDevice,1);

    //set DDC type of DDC1 (for channel 0) to 1 (DDC Bandwidth=24000, SampleRate=32kHz, BitsPerSample=32)
    SetDDC1(hDevice,0,1);

    //retrieve current DDC type of the DDC1
    GetDDC1(hDevice,0,&DDCTypeIndex);
    GetDDCInfo(DDCTypeIndex,&DDCInfo);
    printf("DDC type of DDC1: Index=%u, Bandwidth=%u, SampleRate=%u, BitsPerSample=%u\n",DDCTypeIndex,DDCInfo.Bandwidth,DDCInfo.SampleRate,DDCInfo.BitsPerSample);

    //set DDC type of DDC2 (for channel 0) to 1 (the same as DDC1)
    SetDDC2(hDevice,0,1);

    //retrieve current DDC type of the DDC2
    GetDDC2(hDevice,0,&DDCTypeIndex);
    GetDDCInfo(DDCTypeIndex,&DDCInfo);
    printf("DDC type of DDC2: Index=%u, Bandwidth=%u, SampleRate=%u, BitsPerSample=%u\n",DDCTypeIndex,DDCInfo.Bandwidth,DDCInfo.SampleRate,DDCInfo.BitsPerSample);

    //IFCallbacks is not required in this example
    Callbacks.IFCallback=NULL;

    //DDC1StreamCallback is not required in this example
    Callbacks.DDC1StreamCallback=NULL;

    //DDC1PlaybackStreamCallback is not required in this example
    Callbacks.DDC1PlaybackStreamCallback=NULL;

    //DDC2StreamCallback is not required in this example
    Callbacks.DDC2StreamCallback=NULL;

    //DDC2StreamCallback is not required in this example
    Callbacks.DDC2StreamCallback=NULL;

    //DDC2PreprocessedStreamCallback is used to display signal level in this example
    Callbacks.DDC2PreprocessedStreamCallback=MyDDC2PreprocessedStreamCallback;

    //AudioStreamCallback is used to play audio on wave out device
    Callbacks.AudioStreamCallback=MyAudioStreamCallback;
    
    //AudioPlaybackStreamCallback is not used in this example
    Callbacks.AudioPlaybackStreamCallback=NULL;

    //SweepingCallback is not used in this example
    Callbacks.SweepingCallback=NULL;              

    //register callback functiobs
    SetCallbacks(hDevice,&Callbacks,0);

    //tune channel 0 to an AM station frequency
    SetFrequency(hDevice,0,702000);

    //get absolute frequency where the demodulator of channel 0 is tuned
    GetFrequency(hDevice,0,&Frequency);

    //get front-end frequency
    GetFrontEndFrequency(hDevice,&FrontEndFrequency);

    //get relative DDC1 frequency (channel 0)
    GetDDC1Frequency(hDevice,0,&DDC1Frequency);

    //get relative DDC2 frequency for channel 0
    GetDDC2Frequency(hDevice,0,&DDC2Frequency);

    //get relative demodulator frequency for channel 0
    GetDemodulatorFrequency(hDevice,0,&DemodulatorFrequency);

    printf("Listening to (demodulator absolute frequency): %"PRIu64" Hz\n",Frequency);
    printf("Front-end frequency: %"PRIu64" Hz\n",FrontEndFrequency);
    printf("DDC1 frequency (relative): %d Hz\n",DDC1Frequency);
    printf("DDC2 frequency (relative): %d Hz\n",DDC2Frequency);
    printf("Demodulator frequency (relative): %d Hz\n",DemodulatorFrequency);


    //set AGC params for channel 0, attack=0.02s, decay=0.5s, ref. level=-15dB
    SetAGCParams(hDevice,0,0.020,0.500,-15.0);

    //enable AGC for channel 0
    SetAGC(hDevice,0,1);

    //set AM mode for channel 0
    SetDemodulatorMode(hDevice,0,G39DDC_MODE_AM);

    //set demodulator filter bandwidth for channel 0 to 8kHz
    SetDemodulatorFilterBandwidth(hDevice,0,8000);

    //calculate number of samples per DDC1 buffer (channel 0) - 100 ms
    GetDDC1(hDevice,0,&DDCTypeIndex);
    GetDDCInfo(DDCTypeIndex,&DDCInfo);
    SamplesPerBuffer=100*DDCInfo.SampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
    
    //start DDC1 streaming which has to be running before StartDDC2 is called
    StartDDC1(hDevice,0,SamplesPerBuffer);

    printf("\nPress ENTER to exit.\n\n");    

    //calculate number of samples per DDC2 buffer (channel 0) - 100 ms
    GetDDC2(hDevice,0,&DDCTypeIndex);
    GetDDCInfo(DDCTypeIndex,&DDCInfo);
    SamplesPerBuffer=100*DDCInfo.SampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
               
    //start DDC2 streaming for channel 0, it has to be running before StartAudio is called
    StartDDC2(hDevice,0,SamplesPerBuffer);


    //calculate number of samples per audio buffer (channel 0) - 100 ms
    GetAudio(hDevice,0,&AudioSampleRateIndex);
    GetAudioSampleRate(hDevice,AudioSampleRateIndex,&AudioSampleRate);
    SamplesPerBuffer=100*AudioSampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
    
    //start audio streaming for channel 0
    StartAudio(hDevice,0,SamplesPerBuffer);
    
    getchar();

    //stop audio streaming for channel 0
    StopAudio(hDevice,0);

    //stop DDC2 streaming for channel 0
    StopDDC2(hDevice,0);

    //stop DDC1 streaming for channel 0, it stops DDC2 and audio
    StopDDC1(hDevice,0);
    
    //unregister all callback functions
    SetCallbacks(hDevice,NULL,0);

    SetPower(hDevice,0);

    CloseDevice(hDevice);

    if(PaSimple)
    {
        pa_simple_free(PaSimple);
    }
    
    dlclose(API);

    printf("\n");
    
    return 0;
}