//This example ilustrates how to use object-oriented interface of the G39DDC API
//in non-object C language. It is recomended to use object-oriented interface
//instead of non-object interface in C because it is more efecient than non-object
//interface. Each function of non-object interface verifies device handle
//and translates the handle to reference to internal data structures, methods
//of object interface do not do it.

#include "g39ddcapi.h"
#include <stdio.h>
#include <math.h>
#include <dlfcn.h>
#include <inttypes.h>
#include <pthread.h>
#include <errno.h>
#include <inttypes.h>
#include <pulse/simple.h>
                                       
//prototypes of callback methods
void MyCallback_G39DDC_DDC1StreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,const void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample);
int MyCallback_G39DDC_DDC1PlaybackStreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample);
void MyCallback_G39DDC_DDC2StreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples);
void MyCallback_G39DDC_DDC2PreprocessedStreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS);
void MyCallback_G39DDC_IFCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint16_t MaxADCAmplitude,uint32_t ADCSampleRate);
int MyCallback_G39DDC_SweepingCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint64_t FrontEndFrequency);
void MyCallback_G39DDC_AudioStreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples);
int MyCallback_G39DDC_AudioPlaybackStreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,float *Buffer,uint32_t NumberOfSamples);

IG39DDCDevice *Device;
void *API=NULL;
G39DDCAPI_CREATE_INSTANCE CreateInstance;
pa_simple *PaSimple=NULL;

//table of virtual methods
IG39DDCDeviceCallbackVtbl MyCallbackMethodsTable=
{    
    .G39DDC_DDC1StreamCallback=&MyCallback_G39DDC_DDC1StreamCallback,
    .G39DDC_DDC1PlaybackStreamCallback=&MyCallback_G39DDC_DDC1PlaybackStreamCallback,
    .G39DDC_DDC2StreamCallback=&MyCallback_G39DDC_DDC2StreamCallback,
    .G39DDC_DDC2PreprocessedStreamCallback=&MyCallback_G39DDC_DDC2PreprocessedStreamCallback,
    .G39DDC_IFCallback=&MyCallback_G39DDC_IFCallback,
    .G39DDC_SweepingCallback=&MyCallback_G39DDC_SweepingCallback,
    .G39DDC_AudioStreamCallback=&MyCallback_G39DDC_AudioStreamCallback,
    .G39DDC_AudioPlaybackStreamCallback=&MyCallback_G39DDC_AudioPlaybackStreamCallback
};

//callback object
IG39DDCDeviceCallback MyCallbackObject=
{
    &MyCallbackMethodsTable
};


void MyCallback_G39DDC_DDC1StreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,const void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample)
{
    //it does nothing in this example
}

int MyCallback_G39DDC_DDC1PlaybackStreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample)
{
    //it does nothing in this example
    return 0;
}

void MyCallback_G39DDC_DDC2StreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples)
{
    //it does nothing in this example
}

void MyCallback_G39DDC_DDC2PreprocessedStreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS)
{
 double Slevel_dBm;

    if(Channel==0)
    {
        //convert slevel RMS in Volts to dBm
        Slevel_dBm=10.0*log10(SlevelRMS*SlevelRMS*(1000.0/50.0));

        printf("\rSlevel: RMS[V]=%9.7f V, RMS[dBm]=%6.1f dBm, Peak[V]=%9.7f V",SlevelRMS,Slevel_dBm,SlevelPeak);
        fflush(stdout);
    }
}

void MyCallback_G39DDC_IFCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint16_t MaxADCAmplitude,uint32_t ADCSampleRate)
{
    //it does nothing in this example
}

int MyCallback_G39DDC_SweepingCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint64_t FrontEndFrequency)
{
    //it does nothing in this example
    return 0;
}

void MyCallback_G39DDC_AudioStreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples)
{ 
 int Error;
 
    if(Type==2 && PaSimple!=NULL)
    {
        //we want to play only audio buffers of type 2 which means final audio samples (audio gain, audio filter and volume are applied)
        pa_simple_write(PaSimple,Buffer,NumberOfSamples*2*sizeof(float),&Error);
    }  
}

int MyCallback_G39DDC_AudioPlaybackStreamCallback(IG39DDCDeviceCallback *Callback,IG39DDCDevice *Device,uint32_t Channel,float *Buffer,uint32_t NumberOfSamples)
{
    //it does nothing in this example
    return 0;
}



int  Initialize(void)
{ 
 uint32_t AudioSampleRate;
 uint32_t AudioSampleRateIndex;
 pa_sample_spec SampleFormat;
 int Error;
 
    API=dlopen("libg39ddcapi.so",RTLD_LAZY);

    if(API==NULL)
    {
        printf("Failed to load libg39ddcapi.so. %s\n",dlerror());
        return 0;
    }

    //retrieve addresse of CreateInstance API functions
    CreateInstance=(G39DDCAPI_CREATE_INSTANCE)dlsym(API,"CreateInstance");

    //create instance of device object
    if(!CreateInstance(G39DDC_CLASS_ID_DEVICE,(void**)&Device))
    {
        printf("Failed to create device object. Error code=%u\n",errno);
        dlclose(API);
        return 0;
    }

    //open the first available G39DDC device
    if(!Device->lpVtbl->Open(Device,G39DDC_OPEN_FIRST))
    {
        printf("No available G39DDC device found.\n");
        Device->lpVtbl->Release(Device);
        dlclose(API);
        return 0;
    }
    
    //retrieve current audio sample rate index of channel 0
    Device->lpVtbl->GetAudio(Device,0,&AudioSampleRateIndex);
    Device->lpVtbl->GetAudioSampleRate(Device,AudioSampleRateIndex,&AudioSampleRate);
       
    //initialize pulse audio
    SampleFormat.channels=2;
    SampleFormat.rate=AudioSampleRate;
    SampleFormat.format=PA_SAMPLE_FLOAT32LE;
    
    PaSimple=pa_simple_new(NULL,"G39DDC SDK EXAMPLE",PA_STREAM_PLAYBACK,NULL,"Audio0",&SampleFormat,NULL,NULL,&Error);
    
    if(PaSimple==NULL)
    {
        printf("Failed to initialize pulse audio.\n");
    }
    
    return 1;
}


int main(void)
{
 G39DDC_DDC_INFO DDCInfo;
 uint32_t DDCTypeIndex;
 //G39DDC_CALLBACKS Callbacks;
 uint64_t Frequency;
 uint64_t FrontEndFrequency;
 int32_t DDC1Frequency;
 int32_t DDC2Frequency;
 int32_t DemodulatorFrequency;
 uint32_t SamplesPerBuffer;
 uint32_t AudioSampleRateIndex;
 uint32_t AudioSampleRate;

    if(!Initialize())
    {
        return -1;
    }

    //set power
    Device->lpVtbl->SetPower(Device,1);

    //set DDC type of DDC1 to 1 (DDC Bandwidth=24000, SampleRate=32kHz, BitsPerSample=32)
    Device->lpVtbl->SetDDC1(Device,0,1);

    //retrieve current DDC type of the DDC1
    Device->lpVtbl->GetDDC1(Device,0,&DDCTypeIndex);
    Device->lpVtbl->GetDDCInfo(Device,DDCTypeIndex,&DDCInfo);
    printf("DDC type of DDC1: Index=%u, Bandwidth=%u, SampleRate=%u, BitsPerSample=%u\n",DDCTypeIndex,DDCInfo.Bandwidth,DDCInfo.SampleRate,DDCInfo.BitsPerSample);

    
    //set DDC type of DDC2 to 1 (DDC Bandwidth=24000, SampleRate=32kHz, BitsPerSample=32)
    Device->lpVtbl->SetDDC2(Device,0,1);

    //retrieve current DDC type of the DDC2
    Device->lpVtbl->GetDDC2(Device,0,&DDCTypeIndex);
    Device->lpVtbl->GetDDCInfo(Device,DDCTypeIndex,&DDCInfo);
    printf("DDC type of DDC2: Index=%u, Bandwidth=%u, SampleRate=%u, BitsPerSample=%u\n",DDCTypeIndex,DDCInfo.Bandwidth,DDCInfo.SampleRate,DDCInfo.BitsPerSample);



    //register callback object
    Device->lpVtbl->SetCallback(Device,&MyCallbackObject);

    //tune channel 0 to an AM station frequency
    Device->lpVtbl->SetFrequency(Device,0,702000);

    //get absolute frequency where the demodulator of channel 0 is tuned
    Device->lpVtbl->GetFrequency(Device,0,&Frequency);

    //get front-end frequency
    Device->lpVtbl->GetFrontEndFrequency(Device,&FrontEndFrequency);

    //get DDC1 frequency
    Device->lpVtbl->GetDDC1Frequency(Device,0,&DDC1Frequency);

    //get relative DDC2 frequency for channel 0
    Device->lpVtbl->GetDDC2Frequency(Device,0,&DDC2Frequency);

    //get relative demodulator frequency for channel 0
    Device->lpVtbl->GetDemodulatorFrequency(Device,0,&DemodulatorFrequency);

    printf("Listening to (demodulator absolute frequency): %"PRIu64" Hz\n",Frequency);
    printf("Front-end frequency: %"PRIu64" Hz\n",FrontEndFrequency);
    printf("DDC1 frequency (relative): %d Hz\n",DDC1Frequency);
    printf("DDC2 frequency (relative): %d Hz\n",DDC2Frequency);
    printf("Demodulator frequency (relative): %d Hz\n",DemodulatorFrequency);


    //set AGC params for channel 0, attack=0.02s, decay=0.5s, ref. level=-15dB
    Device->lpVtbl->SetAGCParams(Device,0,0.020,0.500,-15.0);

    //enable AGC for channel 0
    Device->lpVtbl->SetAGC(Device,0,1);

    //set AM mode for channel 0
    Device->lpVtbl->SetDemodulatorMode(Device,0,G39DDC_MODE_AM);

    //set demodulator filter bandwidth for channel 0 to 8kHz
    Device->lpVtbl->SetDemodulatorFilterBandwidth(Device,0,8000);
    
    //calculate number of samples per DDC1 buffer (channel 0) - 100 ms
    Device->lpVtbl->GetDDC1(Device,0,&DDCTypeIndex);
    Device->lpVtbl->GetDDCInfo(Device,DDCTypeIndex,&DDCInfo);
    SamplesPerBuffer=100*DDCInfo.SampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);

    //start DDC1 streaming which has to be running before StartDDC2 is called
    Device->lpVtbl->StartDDC1(Device,0,SamplesPerBuffer);

    printf("\nPress ENTER to exit.\n\n");

    //calculate number of samples per DDC2 buffer (channel 0) - 100 ms
    Device->lpVtbl->GetDDC2(Device,0,&DDCTypeIndex);
    Device->lpVtbl->GetDDCInfo(Device,DDCTypeIndex,&DDCInfo);
    SamplesPerBuffer=100*DDCInfo.SampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
    
    //start DDC2 streaming for channel 0, it has to be running before StartAudio is called
    Device->lpVtbl->StartDDC2(Device,0,SamplesPerBuffer);

    //calculate number of samples per audio buffer (channel 0) - 100 ms
    Device->lpVtbl->GetAudio(Device,0,&AudioSampleRateIndex);
    Device->lpVtbl->GetAudioSampleRate(Device,AudioSampleRateIndex,&AudioSampleRate);
    SamplesPerBuffer=100*AudioSampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
    
    //start audio streaming for channel 0
    Device->lpVtbl->StartAudio(Device,0,SamplesPerBuffer);

    getchar();

    //stop audio streaming for channel 0
    Device->lpVtbl->StopAudio(Device,0);

    //stop DDC2 streaming for channel 0
    Device->lpVtbl->StopDDC2(Device,0);

    //stop DDC1 streaming for channel 0, it stops DDC2 and audio, as well
    Device->lpVtbl->StopDDC1(Device,0);

    
    //unregister callback object
    Device->lpVtbl->SetCallback(Device,NULL);

    Device->lpVtbl->SetPower(Device,0);

    Device->lpVtbl->Release(Device);
    
    if(PaSimple)
    {
        pa_simple_free(PaSimple);
    }

    dlclose(API);

    return 0;
}